@@ -46,6 +46,7 @@ def _log_unrecognized_input(e: Union[str, UnrecognizedInputError]) -> None:
 
 class Command(NamedTuple):
     func: ResolvedNode[Function]
+    namespace: str
     name: str
     decorator: ast.Call
     enabled: str
@@ -178,17 +179,20 @@ class CommandFormatter:
                 if (
                     name == "drgn.commands.command"
                     or name == "drgn.commands.custom_command"
+                    or name == "drgn.commands.raw_command"
                 ):
                     namespace = ""
                 elif (
-                    name == "drgn.commands.linux_kernel_command"
-                    or name == "drgn.commands.linux_kernel_custom_command"
+                    name == "drgn.commands.linux.linux_kernel_command"
+                    or name == "drgn.commands.linux.linux_kernel_custom_command"
+                    or name == "drgn.commands.linux.linux_kernel_raw_command"
                 ):
                     namespace = ""
                     enabled = "linux"
                 elif (
-                    name == "drgn.commands.crash.crash_command"
-                    or name == "drgn.commands.crash.crash_custom_command"
+                    name == "drgn.commands._crash.common.crash_command"
+                    or name == "drgn.commands._crash.common.crash_custom_command"
+                    or name == "drgn.commands._crash.common.crash_raw_command"
                 ):
                     namespace = "crash"
                 else:
@@ -203,6 +207,7 @@ class CommandFormatter:
 
                 command = Command(
                     func=func,
+                    namespace=namespace,
                     name=self._command_name(func.name, decorator, namespace),
                     decorator=decorator,
                     enabled=enabled,
@@ -545,8 +550,16 @@ class CommandFormatter:
                 _format_text(lines, help, indent="    ")
 
     def format(self, command: Command) -> List[str]:
+        label = "drgncommand-"
+        if command.namespace:
+            label = f"{label}{command.namespace}-"
+        label += command.name.replace("*", "asterisk")
+
         command_name = command.name.replace("*", r"\*")
+
         lines = [
+            f".. _{label}:",
+            "",
             command_name,
             "=" * len(command_name),
         ]
@@ -653,7 +666,18 @@ class CommandFormatter:
                     self._format_argument(lines, node)
                 elif type == "drgn.commands.mutually_exclusive_group":
                     for type, node in self._group_arguments(command, node):
-                        visit_positional_argument(type, node)
+                        if type == "drgn.commands.argument":
+                            self._format_argument(lines, node)
+                        elif type == "drgn.commands.argument_group":
+                            _log_unrecognized_input(
+                                "argument_group cannot be child of mutually_exclusive_group"
+                            )
+                        elif type == "drgn.commands.mutually_exclusive_group":
+                            _log_unrecognized_input(
+                                "mutually_exclusive_group cannot be child of mutually_exclusive_group"
+                            )
+                        else:
+                            assert_never(type)
                 elif type == "drgn.commands.argument_group":
                     _log_unrecognized_input(
                         "argument_group cannot be child of argument_group"
