@@ -9,6 +9,8 @@
 
 #include "array.h"
 #include "error.h"
+#include "linux_kernel.h"
+#include "log.h"
 #include "platform.h" // IWYU pragma: associated
 #include "program.h"
 #include "register_state.h"
@@ -68,6 +70,137 @@ static void demangle_cfi_registers_aarch64(struct drgn_program *prog,
 	drgn_register_state_set_from_u64(prog, regs, x30, ra.value);
 }
 
+// elf_gregset_t (in PRSTATUS) and struct user_pt_regs have the same layout.
+// This layout is a prefix of the in-kernel struct pt_regs (but we don't care
+// about any of the extra fields).
+static struct drgn_error *
+get_initial_registers_from_struct_aarch64(struct drgn_program *prog,
+					  const void *buf, size_t size,
+					  struct drgn_register_state **ret)
+{
+	if (size < 272) {
+		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+					 "registers are truncated");
+	}
+
+	struct drgn_register_state *regs =
+		drgn_register_state_create(pstate, true);
+	if (!regs)
+		return &drgn_enomem;
+
+	drgn_register_state_set_from_buffer(regs, pc, (uint64_t *)buf + 32);
+	drgn_register_state_set_from_buffer(regs, sp, (uint64_t *)buf + 31);
+	drgn_register_state_set_range_from_buffer(regs, x19, x30,
+						  (uint64_t *)buf + 19);
+	drgn_register_state_set_range_from_buffer(regs, x0, x18, buf);
+	drgn_register_state_set_from_buffer(regs, pstate, (uint64_t *)buf + 33);
+	drgn_register_state_set_pc_from_register(prog, regs, pc);
+
+	*ret = regs;
+	return NULL;
+}
+
+
+static struct drgn_error *
+get_stackframe_offset(struct drgn_program *prog, uint64_t *ret)
+{
+	if (!prog->aarch64_stackframe_offset_cached) {
+		struct drgn_error *err;
+		struct drgn_qualified_type pt_regs_type;
+
+		err = drgn_program_find_type(prog, "struct pt_regs", NULL, &pt_regs_type);
+		if (err)
+			return err;
+
+		struct drgn_type_member *stackframe_member;
+		uint64_t bit_offset;
+		err = drgn_type_find_member(pt_regs_type.type, "stackframe",
+					    &stackframe_member, &bit_offset);
+		if (err)
+			return err;
+
+		prog->aarch64_stackframe_offset_cached = bit_offset / 8;
+	}
+
+	*ret = prog->aarch64_stackframe_offset_cached;
+	return NULL;
+}
+
+
+static struct drgn_error *
+fallback_unwind_try_pt_regs(struct drgn_program *prog, uint64_t fp,
+			    uint64_t frame[2],
+			    struct drgn_register_state *regs,
+			    struct drgn_register_state **regs_ret,
+			    bool *found_ret)
+{
+	struct drgn_error *err;
+	uint64_t stackframe_offset;
+	const char *method;
+
+	err = get_stackframe_offset(prog, &stackframe_offset);
+	if (err)
+		return err;
+
+	uint64_t pt_regs_addr = fp - stackframe_offset;
+
+	if (!frame[0] && !frame[1]) {
+		// Since c2c6b27b5aa14 ("arm64: stacktrace: unwind exception
+		// boundaries") - v6.13, a frame record with fp and lr both zero
+		// indicates that we can read the next word on the stack for
+		// "metadata" which tells us whether a pt_regs is pushed to the
+		// stack.
+		uint64_t frame_meta_type;
+		err = drgn_program_read_u64(prog, fp + 16, false, &frame_meta_type);
+		if (err)
+			return err;
+
+		// FRAME_META_TYPE_PT_REGS == 2 indicates a pt_regs
+		if (frame_meta_type != 2) {
+			*found_ret = false;
+			return NULL;
+		}
+		method = "frame metadata";
+	} else {
+		// Before this, it's still possible that this stack frame is
+		// embedded within a pt_regs. However, there's no reliable way
+		// to detect it. We can guess what that pt_regs pointer would
+		// be, and check whether any CPU has a __irq_regs value matching
+		// it. If so, we're guaranteed to be in the pt_regs for an IRQ.
+		// If not, it's still possible that this is a pt_regs, but the
+		// __irq_regs accounting has not yet happened (or has been
+		// cleaned up already). So this heuristic shouldn't have any
+		// false positives, but could have some false negatives.
+		bool is_irq_regs;
+		err = drgn_program_is_irq_regs(prog, pt_regs_addr, &is_irq_regs);
+		if (err)
+			return err;
+
+		if (!is_irq_regs) {
+			*found_ret = false;
+			return NULL;
+		}
+		method = "__irq_regs";
+	}
+
+	// Now that we're reasonably confident that there's a pt_regs here, read
+	// it and use it.
+	char pt_regs[272];
+	err = drgn_program_read_memory(prog, &pt_regs, pt_regs_addr,
+				       sizeof(pt_regs), false);
+	if (err)
+		return err;
+	err = get_initial_registers_from_struct_aarch64(prog, &pt_regs,
+							sizeof(pt_regs), regs_ret);
+	if (err)
+		return err;
+
+	drgn_log_debug(prog, "aarch64 fallback: discovered pt_regs 0x%"PRIx64" via %s",
+		       pt_regs_addr, method);
+	*found_ret = true;
+	return NULL;
+}
+
 // Unwind using the frame pointer. Note that leaf functions may not allocate a
 // stack frame, so this may skip the caller of a leaf function. I don't know of
 // a good way around that.
@@ -98,6 +231,24 @@ fallback_unwind_aarch64(struct drgn_program *prog,
 		return err;
 	}
 
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL) {
+		bool found;
+		err = fallback_unwind_try_pt_regs(prog, fp.value, frame, regs, ret, &found);
+		if (err) {
+			// If we encounter any error trying the pt_regs
+			// approach, log it at the debug level and continue.
+			// It's likely an expected error, but could be useful
+			// debugging information.
+			drgn_error_log_debug(prog, err, "encountered error in linux fallback unwinder:");
+			drgn_error_destroy(err);
+			err = NULL;
+		} else if (found) {
+			// We have set the register state based on a pt_regs
+			// frame we found, return!
+			return NULL;
+		}
+	}
+
 	uint64_t unwound_fp =
 		drgn_platform_bswap(&prog->platform) ?
 		bswap_64(frame[0]) : frame[0];
@@ -154,36 +305,6 @@ bad_call_unwind_aarch64(struct drgn_program *prog,
 	return NULL;
 }
 
-// elf_gregset_t (in PRSTATUS) and struct user_pt_regs have the same layout.
-// This layout is a prefix of the in-kernel struct pt_regs (but we don't care
-// about any of the extra fields).
-static struct drgn_error *
-get_initial_registers_from_struct_aarch64(struct drgn_program *prog,
-					  const void *buf, size_t size,
-					  struct drgn_register_state **ret)
-{
-	if (size < 272) {
-		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
-					 "registers are truncated");
-	}
-
-	struct drgn_register_state *regs =
-		drgn_register_state_create(pstate, true);
-	if (!regs)
-		return &drgn_enomem;
-
-	drgn_register_state_set_from_buffer(regs, pc, (uint64_t *)buf + 32);
-	drgn_register_state_set_from_buffer(regs, sp, (uint64_t *)buf + 31);
-	drgn_register_state_set_range_from_buffer(regs, x19, x30,
-						  (uint64_t *)buf + 19);
-	drgn_register_state_set_range_from_buffer(regs, x0, x18, buf);
-	drgn_register_state_set_from_buffer(regs, pstate, (uint64_t *)buf + 33);
-	drgn_register_state_set_pc_from_register(prog, regs, pc);
-
-	*ret = regs;
-	return NULL;
-}
-
 static struct drgn_error *
 pt_regs_get_initial_registers_aarch64(const struct drgn_object *obj,
 				      struct drgn_register_state **ret)
@@ -455,6 +576,15 @@ linux_kernel_pgtable_iterator_next_aarch64(struct drgn_program *prog,
 	uint16_t num_entries = it->last_level_num_entries;
 	uint64_t table = it->it.pgtable;
 	bool table_physical = false;
+	if (table == prog->vmcoreinfo.swapper_pg_dir
+	    && prog->vmcoreinfo.kimage_voffset) {
+		// Avoid recursive address translation on swapper_pg_dir by
+		// directly resolving to a physical address. kimage_voffset has
+		// been present since Linux kernel commit 20a166243328 ("arm64:
+		// kdump: add VMCOREINFO's for user-space tools") (in v4.12).
+		table -= prog->vmcoreinfo.kimage_voffset;
+		table_physical = true;
+	}
 	for (int level = it->levels;; level--) {
 		uint8_t level_shift = page_shift + pgtable_shift * (level - 1);
 		uint16_t index = (virt_addr >> level_shift) & (num_entries - 1);
@@ -500,6 +630,26 @@ linux_kernel_pgtable_iterator_next_aarch64(struct drgn_program *prog,
 	}
 }
 
+static int
+linux_kernel_section_size_bits_fallback_aarch64(struct drgn_program *prog)
+{
+	// Since Linux kernel commit f0b13ee23241 ("arm64/sparsemem: reduce
+	// SECTION_SIZE_BITS") (in v5.12), SECTION_SIZE_BITS is 29 with 64k
+	// pages and 27 otherwise. Linux kernel stable commit 70fd2a63fc1c
+	// ("crash_core, vmcoreinfo: append 'SECTION_SIZE_BITS' to vmcoreinfo")
+	// (in v5.12.13) addresses this, but before that there's no good way to
+	// detect it other than the kernel version.
+	char *p = (char *)prog->vmcoreinfo.osrelease;
+	long major = strtol(p, &p, 10), minor = 0;
+	if (*p == '.')
+		minor = strtol(p + 1, &p, 10);
+	if (major > 5 || (major == 5 && minor >= 12))
+		return prog->vmcoreinfo.page_shift == 16 ? 29 : 27;
+	// Before that, it didn't change since Linux kernel commit 4f04d8f00545
+	// ("arm64: MMU definitions") (in v3.7).
+	return 30;
+}
+
 static uint64_t untagged_addr_aarch64(uint64_t addr)
 {
 	/* Apply TBI by sign extending bit 55 into bits 56-63. */
@@ -530,5 +680,9 @@ const struct drgn_architecture_info arch_info_aarch64 = {
 		linux_kernel_pgtable_iterator_init_aarch64,
 	.linux_kernel_pgtable_iterator_next =
 		linux_kernel_pgtable_iterator_next_aarch64,
+	.linux_kernel_section_size_bits_fallback =
+		linux_kernel_section_size_bits_fallback_aarch64,
+	// MAX_PHYSMEM_BITS depends on CONFIG_ARM64_PA_BITS. Use the default
+	// fallback.
 	.untagged_addr = untagged_addr_aarch64,
 };
