@@ -101,6 +101,17 @@ invalid:
 	}
 	free(line);
 	fclose(file);
+
+	// Depending on sysctl kptr_restrict and perf_event_paranoid, only a
+	// user with CAP_SYSLOG can access the kallsyms addresses. We're not
+	// guaranteed to be running as root even in the live case. Given that
+	// this function is mainly used as a fallback for specific symbols, we
+	// can be reasonably confident that the expected address is not zero.
+	// Catch the case where we find the symbol and read address zero, and
+	// raise an error that will hopefully be useful to the user.
+	if (!err && *ret == 0 && geteuid() != 0)
+		return drgn_error_create(DRGN_ERROR_OTHER,
+					 "drgn requires root to access /proc/kallsyms");
 	return err;
 }
 
@@ -1666,7 +1677,8 @@ yield_vmlinux(struct linux_kernel_loaded_module_iterator *it,
 		drgn_log_debug(prog,
 			       "found kernel build ID %s in VMCOREINFO",
 			       module->build_id_str);
-	} else if (prog->flags & DRGN_PROGRAM_IS_LIVE) {
+	} else if ((prog->flags & (DRGN_PROGRAM_IS_LIVE | DRGN_PROGRAM_IS_LOCAL))
+		   == (DRGN_PROGRAM_IS_LIVE | DRGN_PROGRAM_IS_LOCAL)) {
 		// Before that, on the live kernel, we can get the build ID from
 		// /sys/kernel/notes.
 		_cleanup_free_ void *build_id_buf = NULL;
@@ -2340,7 +2352,8 @@ kernel_module_find_or_create_internal(const struct drgn_object *module_ptr,
 	// be disabled if we encounter permission issues using
 	// /sys/module/$module/sections.
 	bool use_sys_module = false;
-	if (prog->flags & DRGN_PROGRAM_IS_LOCAL) {
+	if ((prog->flags & (DRGN_PROGRAM_IS_LIVE | DRGN_PROGRAM_IS_LOCAL))
+	    == (DRGN_PROGRAM_IS_LIVE | DRGN_PROGRAM_IS_LOCAL)) {
 		char *env = getenv("DRGN_USE_SYS_MODULE");
 		use_sys_module = !env || atoi(env);
 	}
