@@ -213,6 +213,9 @@ typedef struct drgn_error *
  *   - @ref linux_kernel_pgtable_iterator_destroy
  *   - @ref linux_kernel_pgtable_iterator_init
  *   - @ref linux_kernel_pgtable_iterator_next
+ * - Define the SPARSEMEM constant fallback getters if applicable:
+ *   - @ref linux_kernel_section_size_bits_fallback
+ *   - @ref linux_kernel_max_physmem_bits_fallback
  * - Add the architecture name to `HAVE_FULL_MM_SUPPORT` in
  *   `tests/linux_kernel/__init__.py`.
  * - Update `docs/support_matrix.rst`.
@@ -431,6 +434,15 @@ struct drgn_architecture_info {
 	struct drgn_error *(*linux_kernel_live_direct_mapping_fallback)(struct drgn_program *prog,
 									uint64_t *address_ret,
 									uint64_t *size_ret);
+	/**
+	 * Return the address of the direct mapping virtual address range.
+	 *
+	 * This hook is used for @ref linux_helper_direct_mapping_offset(). It
+	 * is optional: if not provided, we can infer it using a page table walk
+	 * and a symbol from the direct mapping.
+	 */
+	struct drgn_error *(*linux_kernel_direct_mapping_offset)(struct drgn_program *prog,
+								 uint64_t *address_ret);
 	/** Allocate a Linux kernel page table iterator. */
 	struct drgn_error *(*linux_kernel_pgtable_iterator_create)(struct drgn_program *,
 								   struct pgtable_iterator **);
@@ -463,6 +475,20 @@ struct drgn_architecture_info {
 	 * @see pgtable_iterator_next_fn
 	 */
 	pgtable_iterator_next_fn *linux_kernel_pgtable_iterator_next;
+	/**
+	 * Get the value of `SECTION_SIZE_BITS` for Linux kernel versions before
+	 * v5.13.
+	 */
+	int (*linux_kernel_section_size_bits_fallback)(struct drgn_program *);
+	/**
+	 * Get the value of `MAX_PHYSMEM_BITS` for Linux kernel versions before
+	 * v5.9.
+	 *
+	 * If this is omitted but @ref linux_kernel_section_size_bits_fallback
+	 * is defined, then this will be computed from `NR_SECTION_ROOTS` and
+	 * `SECTION_SIZE_BITS` instead.
+	 */
+	int (*linux_kernel_max_physmem_bits_fallback)(struct drgn_program *);
 	/**
 	 * Return the canonical form of a virtual address, i.e. apply any
 	 * transformations that the CPU applies to the address before page
