@@ -673,8 +673,7 @@ drgn_program_set_core_dump_fd_internal(struct drgn_program *prog, int fd,
 	return NULL;
 
 out_segments:
-	drgn_memory_reader_deinit(&prog->reader);
-	drgn_memory_reader_init(&prog->reader);
+	drgn_memory_reader_clear(&prog->reader);
 	free(prog->file_segments);
 	prog->file_segments = NULL;
 out_notes:
@@ -733,6 +732,68 @@ drgn_program_set_kernel(struct drgn_program *prog)
 	return drgn_program_set_core_dump(prog, "/proc/kcore");
 }
 
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_program_set_linux_kernel_custom(struct drgn_program *prog,
+				     const char *vmcoreinfo,
+				     size_t vmcoreinfo_size, bool is_live)
+{
+	struct drgn_error *err;
+
+	if (!prog->has_platform) {
+		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+			"platform must be set before calling set_linux_kernel_custom()");
+	}
+
+	if (prog->flags & DRGN_PROGRAM_IS_LINUX_KERNEL)
+		return NULL;
+
+	// Parse vmcoreinfo if not already set via Program constructor
+	bool had_vmcoreinfo = prog->vmcoreinfo.raw != NULL;
+	if (!had_vmcoreinfo) {
+		err = drgn_program_parse_vmcoreinfo(prog, vmcoreinfo,
+						    vmcoreinfo_size);
+		if (err)
+			return err;
+	}
+
+	/*
+	 * Register a virtual memory reader that uses page table walking.
+	 * This translates virtual addresses to physical using swapper_pg_dir
+	 * from vmcoreinfo, then reads physical memory from user-registered
+	 * segments.
+	 */
+	if (prog->platform.arch->linux_kernel_pgtable_iterator_next) {
+		err = drgn_program_add_memory_segment(prog, 0, UINT64_MAX,
+						      read_memory_via_pgtable,
+						      prog, false);
+		if (err)
+			goto out_vmcoreinfo;
+	}
+
+	enum drgn_program_flags old_flags = prog->flags;
+	prog->flags |= DRGN_PROGRAM_IS_LINUX_KERNEL;
+	if (is_live)
+		prog->flags |= DRGN_PROGRAM_IS_LIVE;
+
+	err = drgn_program_finish_set_kernel(prog);
+	if (err) {
+		prog->flags = old_flags;
+		drgn_memory_reader_clear_virtual(&prog->reader);
+		goto out_vmcoreinfo;
+	}
+
+	drgn_call_plugins_prog("drgn_prog_set", prog);
+	return NULL;
+
+out_vmcoreinfo:
+	// Free vmcoreinfo buffer if it was not provided by the caller
+	if (!had_vmcoreinfo) {
+		free(prog->vmcoreinfo.raw);
+		memset(&prog->vmcoreinfo, 0, sizeof(prog->vmcoreinfo));
+	}
+	return err;
+}
+
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_program_set_pid(struct drgn_program *prog, pid_t pid)
 {
@@ -776,8 +837,7 @@ drgn_program_set_pid(struct drgn_program *prog, pid_t pid)
 	return NULL;
 
 out_segments:
-	drgn_memory_reader_deinit(&prog->reader);
-	drgn_memory_reader_init(&prog->reader);
+	drgn_memory_reader_clear(&prog->reader);
 	free(prog->file_segments);
 	prog->file_segments = NULL;
 out_fd:
@@ -1423,14 +1483,18 @@ drgn_program_kernel_get_crashed_cpu(struct drgn_program *prog, uint64_t *ret)
 		if (err)
 			return err;
 		err = drgn_object_read_integer(&cpu, &cpu_value);
-		if (!err)
-			*ret = cpu_value.uvalue;
-	} else if (err->code == DRGN_ERROR_LOOKUP) {
+		if (err)
+			return err;
+		if (cpu_value.svalue == -1) {
+			return drgn_error_create(DRGN_ERROR_LOOKUP,
+						 "panic_cpu is not set");
+		}
+		*ret = cpu_value.uvalue;
+	} else if (drgn_error_catch(&err, DRGN_ERROR_LOOKUP)) {
 		// On x86 and x86-64 only, the crashed CPU is also in an int
 		// crashing_cpu. Use this as a fallback for kernels before
 		// commit 1717f2096b54 ("panic, x86: Fix re-entrance problem due
 		// to panic on NMI") (in v4.5).
-		drgn_error_destroy(err);
 		err = drgn_program_find_object(prog, "crashing_cpu", NULL,
 					       DRGN_FIND_OBJECT_VARIABLE, &cpu);
 		if (!err) {
@@ -1445,12 +1509,10 @@ drgn_program_kernel_get_crashed_cpu(struct drgn_program *prog, uint64_t *ret)
 				*ret = 0;
 			else
 				*ret = cpu_value.uvalue;
-		} else if (err->code == DRGN_ERROR_LOOKUP) {
+		} else if (drgn_error_catch(&err, DRGN_ERROR_LOOKUP)) {
 			// Before Linux kernel commit 5bc329503e81 ("x86/mce:
 			// Handle broadcasted MCE gracefully with kexec") (in
 			// v4.12), crashing_cpu is only defined in SMP kernels.
-			drgn_error_destroy(err);
-			err = NULL;
 			*ret = 0;
 		}
 	}
