@@ -32,7 +32,27 @@ static inline PyObject *PyObject_CallOneArg(PyObject *callable, PyObject *arg)
 }
 #endif
 
+#if PY_VERSION_HEX < 0x030a00a3
+static inline int PyModule_AddObjectRef(PyObject *mod, const char *name,
+					PyObject *value)
+{
+	Py_XINCREF(value);
+	int ret = PyModule_AddObject(mod, name, value);
+	if (ret)
+		Py_XDECREF(value);
+	return ret;
+}
+#endif
+
 #if PY_VERSION_HEX < 0x030d00a1
+static inline int PyModule_Add(PyObject *mod, const char *name, PyObject *value)
+{
+	int ret = PyModule_AddObject(mod, name, value);
+	if (ret)
+		Py_XDECREF(value);
+	return ret;
+}
+
 #define PyThreadState_GetUnchecked _PyThreadState_UncheckedGet
 #endif
 
@@ -42,32 +62,38 @@ static inline PyObject *PyObject_CallOneArg(PyObject *callable, PyObject *arg)
 
 #define DRGNPY_PUBLIC __attribute__((__visibility__("default")))
 
-// PyLong_From* and PyLong_As* for stdint.h types. These use _Generic for
-// slightly more type safety (e.g., so you can't pass an int64_t to
-// PyLong_FromUint64()).
-#if ULONG_MAX == UINT64_MAX
-#define PyLong_FromUint64(v) _Generic((v), uint64_t: PyLong_FromUnsignedLong)(v)
-#define PyLong_AsUint64(obj) ((uint64_t)PyLong_AsUnsignedLong(obj))
-#define PyLong_AsUint64Mask(obj) ((uint64_t)PyLong_AsUnsignedLongMask(obj))
-#elif ULLONG_MAX == UINT64_MAX
-#define PyLong_FromUint64(v) _Generic((v), uint64_t: PyLong_FromUnsignedLongLong)(v)
-#define PyLong_AsUint64(obj) ((uint64_t)PyLong_AsUnsignedLongLong(obj))
-#define PyLong_AsUint64Mask(obj) ((uint64_t)PyLong_AsUnsignedLongLongMask(obj))
+// Added in Python 3.14. We provide a fallback for older versions.
+int PyLong_IsNegative(PyObject *obj);
+
+#if PY_VERSION_HEX < 0x030e00a1
+static inline PyObject *PyLong_FromInt64(int64_t value)
+{
+	return PyLong_FromLongLong(value);
+}
+static inline PyObject *PyLong_FromUInt32(uint32_t value)
+{
+	return PyLong_FromUnsignedLong(value);
+}
+static inline PyObject *PyLong_FromUInt64(uint64_t value)
+{
+	return PyLong_FromUnsignedLongLong(value);
+}
+
+int PyLong_AsInt64(PyObject *obj, int64_t *value);
+int PyLong_AsUInt32(PyObject *obj, uint32_t *value);
+int PyLong_AsUInt64(PyObject *obj, uint64_t *value);
 #endif
 
-#if LONG_MIN == INT64_MIN && LONG_MAX == INT64_MAX
-#define PyLong_FromInt64(v) _Generic((v), int64_t: PyLong_FromLong)(v)
-#define PyLong_AsInt64(obj) ((int64_t)PyLong_AsLong(obj))
-#elif LLONG_MIN == INT64_MIN && LLONG_MAX == INT64_MAX
-#define PyLong_FromInt64(v) _Generic((v), int64_t: PyLong_FromLongLong)(v)
-#define PyLong_AsInt64(obj) ((int64_t)PyLong_AsLongLong(obj))
-#endif
-
-#if ULONG_MAX >= UINT32_MAX
-#define PyLong_FromUint32(v) _Generic((v), uint32_t: PyLong_FromUnsignedLong)(v)
-#define PyLong_FromUint16(v) _Generic((v), uint16_t: PyLong_FromUnsignedLong)(v)
-#define PyLong_FromUint8(v) _Generic((v), uint8_t: PyLong_FromUnsignedLong)(v)
-#endif
+// These variants don't exist as of Python 3.14.
+static inline PyObject *PyLong_FromUInt8(uint8_t value)
+{
+	return PyLong_FromUnsignedLong(value);
+}
+static inline PyObject *PyLong_FromUInt16(uint16_t value)
+{
+	return PyLong_FromUnsignedLong(value);
+}
+int PyLong_AsUInt16(PyObject *obj, uint16_t *value);
 
 #define Py_RETURN_BOOL(cond) do {	\
 	if (cond)			\
@@ -137,6 +163,11 @@ typedef struct {
 	const struct drgn_language *language;
 } Language;
 
+typedef struct {
+	PyObject_HEAD
+	struct drgn_memory_search_iterator *it;
+} MemorySearchIterator;
+
 typedef struct {
 	PyObject_HEAD
 	struct drgn_module *module;
@@ -160,7 +191,8 @@ typedef struct {
 typedef struct {
 	PyObject_HEAD
 	DrgnObject *obj;
-	uint64_t length, index;
+	uint64_t index, end;
+	int step;
 } ObjectIterator;
 
 typedef struct {
@@ -206,6 +238,11 @@ typedef struct {
 	const struct drgn_register *reg;
 } Register;
 
+typedef struct {
+	PyObject_HEAD
+	struct drgn_source_location_list *locs;
+} SourceLocationList;
+
 typedef struct {
 	PyObject_HEAD
 	struct drgn_stack_trace *trace;
@@ -290,6 +327,10 @@ extern PyTypeObject ExtraModule_type;
 extern PyTypeObject FaultError_type;
 extern PyTypeObject Language_type;
 extern PyTypeObject MainModule_type;
+extern PyTypeObject MemorySearchIteratorWithBytes_type;
+extern PyTypeObject MemorySearchIteratorWithInt_type;
+extern PyTypeObject MemorySearchIteratorWithStr_type;
+extern PyTypeObject MemorySearchIterator_type;
 extern PyTypeObject ModuleIteratorWithNew_type;
 extern PyTypeObject ModuleIterator_type;
 extern PyTypeObject ModuleSectionAddressesIterator_type;
@@ -301,6 +342,8 @@ extern PyTypeObject Program_type;
 extern PyTypeObject Register_type;
 extern PyTypeObject RelocatableModule_type;
 extern PyTypeObject SharedLibraryModule_type;
+extern PyTypeObject SourceLocationList_type;
+extern PyObject *SourceLocation_type;
 extern PyTypeObject StackFrame_type;
 extern PyTypeObject StackTrace_type;
 extern PyTypeObject SymbolIndex_type;
@@ -329,6 +372,17 @@ int init_logging(void);
 
 bool set_drgn_in_python(void);
 void clear_drgn_in_python(void);
+
+static inline void drgn_in_python_cleanup(bool *clearp)
+{
+	if (*clearp)
+		clear_drgn_in_python();
+}
+
+#define drgn_in_python_guard()							\
+	__attribute__((__cleanup__(drgn_in_python_cleanup), __unused__))	\
+	bool PP_UNIQUE(clear) = set_drgn_in_python()
+
 struct drgn_error *drgn_error_from_python(void);
 void *set_drgn_error(struct drgn_error *err);
 void *set_error_type_name(const char *format,
@@ -336,6 +390,9 @@ void *set_error_type_name(const char *format,
 
 #define call_tp_alloc(type) ((type *)type##_type.tp_alloc(&type##_type, 0))
 
+PyObject *MemorySearchIterator_wrap(PyTypeObject *type,
+				    struct drgn_memory_search_iterator *it);
+
 PyObject *Module_wrap(struct drgn_module *module);
 static inline Program *Module_prog(Module *module)
 {
@@ -383,6 +440,9 @@ Program *program_from_core_dump(PyObject *self, PyObject *args, PyObject *kwds);
 Program *program_from_kernel(PyObject *self);
 Program *program_from_pid(PyObject *self, PyObject *args, PyObject *kwds);
 
+int add_SourceLocation(PyObject *m);
+PyObject *SourceLocationList_wrap(struct drgn_source_location_list *locs);
+
 PyObject *Symbol_wrap(struct drgn_symbol *sym, PyObject *name_obj);
 PyObject *Symbol_list_wrap(struct drgn_symbol **symbols, size_t count,
 			   PyObject *name_obj);
