@@ -2,8 +2,13 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include <elfutils/libdwfl.h>
+#ifdef WITH_PCRE2
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
+#endif
 
 #include "drgnpy.h"
+#include "../error.h"
 #include "../path.h"
 
 /* Basically PyModule_AddType(), which is only available since Python 3.9. */
@@ -16,21 +21,12 @@ static int add_type(PyObject *module, PyTypeObject *type)
 	const char *dot = strrchr(type->tp_name, '.');
 	if (dot)
 		name = dot + 1;
-	Py_INCREF(type);
-	ret = PyModule_AddObject(module, name, (PyObject *)type);
-	if (ret)
-		Py_DECREF(type);
-	return ret;
+	return PyModule_AddObjectRef(module, name, (PyObject *)type);
 }
 
 static int add_bool(PyObject *module, const char *name, bool value)
 {
-	PyObject *obj = value ? Py_True : Py_False;
-	Py_INCREF(obj);
-	int ret = PyModule_AddObject(module, name, obj);
-	if (ret)
-		Py_DECREF(obj);
-	return ret;
+	return PyModule_AddObjectRef(module, name, value ? Py_True : Py_False);
 }
 
 PyObject *MissingDebugInfoError;
@@ -38,7 +34,7 @@ static PyObject *NoDefaultProgramError;
 PyObject *ObjectAbsentError;
 PyObject *OutOfBoundsError;
 
-static _Thread_local PyObject *default_prog;
+static _Thread_local Program *default_prog;
 
 static PyObject *get_default_prog(PyObject *self, PyObject *_)
 {
@@ -47,7 +43,7 @@ static PyObject *get_default_prog(PyObject *self, PyObject *_)
 		return NULL;
 	}
 	Py_INCREF(default_prog);
-	return default_prog;
+	return (PyObject *)default_prog;
 }
 
 static PyObject *set_default_prog(PyObject *self, PyObject *arg)
@@ -56,7 +52,7 @@ static PyObject *set_default_prog(PyObject *self, PyObject *arg)
 		Py_CLEAR(default_prog);
 	} else if (PyObject_TypeCheck(arg, &Program_type)) {
 		Py_INCREF(arg);
-		Py_XSETREF(default_prog, arg);
+		Py_XSETREF(default_prog, (Program *)arg);
 	} else {
 		PyErr_SetString(PyExc_TypeError,
 				"prog must be Program or None");
@@ -106,32 +102,87 @@ static PyObject *sizeof_(PyObject *self, PyObject *arg)
 		err = drgn_type_sizeof(((DrgnType *)arg)->type, &size);
 	} else if (PyObject_TypeCheck(arg, &DrgnObject_type)) {
 		err = drgn_object_sizeof(&((DrgnObject *)arg)->obj, &size);
+	} else if (PyUnicode_Check(arg)) {
+		if (!default_prog) {
+			PyErr_SetString(NoDefaultProgramError,
+					"no default program");
+			return NULL;
+		}
+
+		const char *name = PyUnicode_AsUTF8(arg);
+		if (!name)
+			return NULL;
+
+		drgn_in_python_guard();
+		struct drgn_qualified_type qualified_type;
+		err = drgn_program_find_type(&default_prog->prog, name, NULL,
+					     &qualified_type);
+		if (!err) {
+			err = drgn_type_sizeof(qualified_type.type, &size);
+		} else if (drgn_error_catch(&err, DRGN_ERROR_LOOKUP)) {
+			DRGN_OBJECT(obj, &default_prog->prog);
+			err = drgn_program_find_object(&default_prog->prog,
+						       name, NULL,
+						       DRGN_FIND_OBJECT_ANY,
+						       &obj);
+			if (!err)
+				err = drgn_object_sizeof(&obj, &size);
+		}
 	} else {
 		return PyErr_Format(PyExc_TypeError,
-				    "expected Type or Object, not %s",
+				    "expected Type, Object, or str, not %s",
 				    Py_TYPE(arg)->tp_name);
 	}
 	if (err)
 		return set_drgn_error(err);
-	return PyLong_FromUint64(size);
+	return PyLong_FromUInt64(size);
+}
+
+static bool
+default_prog_find_type(PyObject *arg, struct drgn_qualified_type *ret)
+{
+	struct drgn_error *err;
+
+	if (!default_prog) {
+		PyErr_SetString(NoDefaultProgramError, "no default program");
+		return false;
+	}
+
+	const char *name = PyUnicode_AsUTF8(arg);
+	if (!name)
+		return false;
+
+	bool clear = set_drgn_in_python();
+	err = drgn_program_find_type(&default_prog->prog, name, NULL, ret);
+	if (clear)
+		clear_drgn_in_python();
+	if (err) {
+		set_drgn_error(err);
+		return false;
+	}
+	return true;
 }
 
 static PyObject *alignof_(PyObject *self, PyObject *arg)
 {
 	struct drgn_error *err;
-	if (!PyObject_TypeCheck(arg, &DrgnType_type)) {
-		return PyErr_Format(PyExc_TypeError, "expected Type not %s",
+	struct drgn_qualified_type qualified_type;
+	if (PyObject_TypeCheck(arg, &DrgnType_type)) {
+		qualified_type.type = ((DrgnType *)arg)->type;
+		qualified_type.qualifiers = ((DrgnType *)arg)->qualifiers;
+	} else if (PyUnicode_Check(arg)) {
+		if (!default_prog_find_type(arg, &qualified_type))
+			return NULL;
+	} else {
+		return PyErr_Format(PyExc_TypeError,
+				    "expected Type or str, not %s",
 				    Py_TYPE(arg)->tp_name);
 	}
-	struct drgn_qualified_type qualified_type = {
-		.type = ((DrgnType *)arg)->type,
-		.qualifiers = ((DrgnType *)arg)->qualifiers,
-	};
 	uint64_t size;
 	err = drgn_type_alignof(qualified_type, &size);
 	if (err)
 		return set_drgn_error(err);
-	return PyLong_FromUint64(size);
+	return PyLong_FromUInt64(size);
 }
 
 static PyObject *offsetof_(PyObject *self, PyObject *args, PyObject *kwds)
@@ -139,16 +190,31 @@ static PyObject *offsetof_(PyObject *self, PyObject *args, PyObject *kwds)
 	struct drgn_error *err;
 
 	static char *keywords[] = {"type", "member", NULL};
-	DrgnType *type;
+	PyObject *arg;
 	const char *member;
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!s:offsetof", keywords,
-					 &DrgnType_type, &type, &member))
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "Os:offsetof", keywords,
+					 &arg, &member))
 		return NULL;
+
+	struct drgn_type *type;
+	if (PyObject_TypeCheck(arg, &DrgnType_type)) {
+		type = ((DrgnType *)arg)->type;
+	} else if (PyUnicode_Check(arg)) {
+		struct drgn_qualified_type qualified_type;
+		if (!default_prog_find_type(arg, &qualified_type))
+			return NULL;
+		type = qualified_type.type;
+	} else {
+		return PyErr_Format(PyExc_TypeError,
+				    "expected Type or str, not %s",
+				    Py_TYPE(arg)->tp_name);
+	}
+
 	uint64_t offset;
-	err = drgn_type_offsetof(type->type, member, &offset);
+	err = drgn_type_offsetof(type, member, &offset);
 	if (err)
 		return set_drgn_error(err);
-	return PyLong_FromUint64(offset);
+	return PyLong_FromUInt64(offset);
 }
 
 static PyMethodDef drgn_methods[] = {
@@ -254,27 +320,20 @@ static int add_type_aliases(PyObject *m)
 	if (!typing_Union)
 		return -1;
 
-	PyObject *typing_SupportsIndex =
-		PyObject_GetAttrString(typing_module, "SupportsIndex");
-	if (PyModule_AddObject(m, "IntegerLike", typing_SupportsIndex) == -1) {
-		Py_XDECREF(typing_SupportsIndex);
+	if (PyModule_Add(m, "IntegerLike",
+			 PyObject_GetAttrString(typing_module, "SupportsIndex")))
 		return -1;
-	}
 
 	_cleanup_pydecref_ PyObject *item =
 		Py_BuildValue("OOO", &PyUnicode_Type, &PyBytes_Type,
 			      os_PathLike);
 	if (!item)
 		return -1;
-	PyObject *Path = PyObject_GetItem(typing_Union, item);
-	if (!Path)
+	if (PyModule_Add(m, "Path", PyObject_GetItem(typing_Union, item)))
 		return -1;
-	if (PyModule_AddObject(m, "Path", Path) == -1) {
-		Py_DECREF(Path);
-		return -1;
-	}
 
-	return 0;
+	return PyModule_Add(m, "MemorySearchIterator",
+			    PyObject_GetAttrString(typing_module, "Iterator"));
 }
 
 PyMODINIT_FUNC PyInit__drgn(void); // Silence -Wmissing-prototypes.
@@ -288,15 +347,17 @@ DRGNPY_PUBLIC PyMODINIT_FUNC PyInit__drgn(void)
 		name = PyErr_NewExceptionWithDoc("_drgn." #name,		\
 						 drgn_##name##_DOC, NULL,	\
 						 NULL);				\
-		if (name && PyModule_AddObject(m, #name, name))			\
-			Py_CLEAR(name);						\
-		!name;								\
+		!name || PyModule_AddObjectRef(m, #name, name);			\
 	})
 
 	if (add_module_constants(m) ||
 	    add_type(m, &DebugInfoOptions_type) ||
 	    add_type(m, &Language_type) || add_languages() ||
 	    add_type(m, &DrgnObject_type) ||
+	    PyType_Ready(&MemorySearchIteratorWithBytes_type) ||
+	    PyType_Ready(&MemorySearchIteratorWithInt_type) ||
+	    PyType_Ready(&MemorySearchIteratorWithStr_type) ||
+	    PyType_Ready(&MemorySearchIterator_type) ||
 	    add_type(m, &Module_type) ||
 	    add_type(m, &MainModule_type) ||
 	    add_type(m, &SharedLibraryModule_type) ||
@@ -312,6 +373,8 @@ DRGNPY_PUBLIC PyMODINIT_FUNC PyInit__drgn(void)
 	    add_type(m, &Platform_type) ||
 	    add_type(m, &Program_type) ||
 	    add_type(m, &Register_type) ||
+	    add_type(m, &SourceLocationList_type) ||
+	    add_SourceLocation(m) ||
 	    add_type(m, &StackFrame_type) ||
 	    add_type(m, &StackTrace_type) ||
 	    add_type(m, &Symbol_type) ||
@@ -346,13 +409,9 @@ DRGNPY_PUBLIC PyMODINIT_FUNC PyInit__drgn(void)
 	if (add_type(m, &ObjectNotFoundError_type))
 		goto err;
 
-	PyObject *host_platform_obj = Platform_wrap(&drgn_host_platform);
-	if (!host_platform_obj)
+	if (PyModule_Add(m, "host_platform",
+			 Platform_wrap(&drgn_host_platform)))
 		goto err;
-	if (PyModule_AddObject(m, "host_platform", host_platform_obj)) {
-		Py_DECREF(host_platform_obj);
-		goto err;
-	}
 
 	if (PyModule_AddStringConstant(m, "_elfutils_version",
 				       dwfl_version(NULL)))
@@ -388,6 +447,26 @@ DRGNPY_PUBLIC PyMODINIT_FUNC PyInit__drgn(void)
 		    ))
 		goto err;
 
+	if (add_bool(m, "_with_pcre2",
+#ifdef WITH_PCRE2
+		     true
+#else
+		     false
+#endif
+		    ))
+		goto err;
+
+	// Allow tests to detect whether PCRE2 supports UTF-8 patterns within
+	// invalid UTF.
+	if (add_bool(m, "_with_pcre2_utf",
+#if WITH_PCRE2 && (PCRE2_MAJOR > 10 || (PCRE2_MAJOR == 10 && PCRE2_MINOR >= 34))
+		     true
+#else
+		     false
+#endif
+		    ))
+		goto err;
+
 	return m;
 
 err:
