@@ -18,22 +18,18 @@ static int DrgnObject_literal(struct drgn_object *res, PyObject *literal)
 	if (PyBool_Check(literal)) {
 		err = drgn_object_bool_literal(res, literal == Py_True);
 	} else if (PyLong_Check(literal)) {
-		bool is_negative = false;
-		uint64_t uvalue = PyLong_AsUint64(literal);
-
-		/* Assume an overflow is due to a negative number and retry */
-		if (uvalue == (uint64_t)-1 && PyErr_Occurred() &&
-		    PyErr_ExceptionMatches(PyExc_OverflowError)) {
-			is_negative = true;
-			PyErr_Clear();
+		bool is_negative = PyLong_IsNegative(literal);
+		uint64_t uvalue;
+		if (is_negative) {
 			_cleanup_pydecref_ PyObject *negated =
 				PyNumber_Negative(literal);
 			if (!negated)
 				return -1;
-			uvalue = PyLong_AsUint64(negated);
-		}
-		if (uvalue == (uint64_t)-1 && PyErr_Occurred())
+			if (PyLong_AsUInt64(negated, &uvalue))
+				return -1;
+		} else if (PyLong_AsUInt64(literal, &uvalue)) {
 			return -1;
+		}
 		err = drgn_object_integer_literal(res, uvalue);
 		if (!err && is_negative)
 			err = drgn_object_neg(res, res);
@@ -112,19 +108,13 @@ static int serialize_compound_value(struct drgn_program *prog, char *buf,
 		return -1;
 	}
 
-	_cleanup_pydecref_ PyObject *tmp = PyMapping_Items(value_obj);
-	if (!tmp)
-		return -1;
-	// Since Python 3.7, PyMapping_Items() always returns a list. However,
-	// before that, it could also return a tuple.
-	_cleanup_pydecref_ PyObject *items =
-		PySequence_Fast(tmp, "items must be sequence");
+	_cleanup_pydecref_ PyObject *items = PyMapping_Items(value_obj);
 	if (!items)
 		return -1;
 
-	Py_ssize_t num_items = PySequence_Fast_GET_SIZE(items);
+	Py_ssize_t num_items = PyList_GET_SIZE(items);
 	for (Py_ssize_t i = 0; i < num_items; i++) {
-		PyObject *item = PySequence_Fast_GET_ITEM(items, i);
+		PyObject *item = PyList_GET_ITEM(items, i);
 		if (!PyTuple_Check(item) || PyTuple_GET_SIZE(item) != 2) {
 			PyErr_SetString(PyExc_TypeError, "invalid item");
 			return -1;
@@ -246,13 +236,14 @@ static int serialize_py_object(struct drgn_program *prog, char *buf,
 			PyNumber_Long(value_obj);
 		if (!long_obj)
 			return -1;
+		unsigned long long lvalue =
+			PyLong_AsUnsignedLongLongMask(long_obj);
+		if (lvalue == (unsigned long long)-1 && PyErr_Occurred())
+			return -1;
 		union {
 			int64_t svalue;
 			uint64_t uvalue;
-		} tmp;
-		tmp.uvalue = PyLong_AsUint64Mask(long_obj);
-		if (tmp.uvalue == (uint64_t)-1 && PyErr_Occurred())
-			return -1;
+		} tmp = { .uvalue = lvalue };
 		if (type->encoding == DRGN_OBJECT_ENCODING_SIGNED) {
 			tmp.svalue = truncate_signed(tmp.svalue,
 						     type->bit_size);
@@ -478,14 +469,14 @@ static DrgnObject *DrgnObject_new(PyTypeObject *subtype, PyObject *args,
 				PyNumber_Long(value_obj);
 			if (!long_obj)
 				return NULL;
+			unsigned long long lvalue =
+				PyLong_AsUnsignedLongLongMask(long_obj);
+			if (lvalue == (unsigned long long)-1 && PyErr_Occurred())
+				return NULL;
 			union {
 				int64_t svalue;
 				uint64_t uvalue;
-			} tmp = {
-				.uvalue = PyLong_AsUint64Mask(long_obj)
-			};
-			if (tmp.uvalue == (uint64_t)-1 && PyErr_Occurred())
-				return NULL;
+			} tmp = { .uvalue = lvalue };
 			if (object_type.encoding == DRGN_OBJECT_ENCODING_SIGNED) {
 				err = drgn_object_set_signed_internal(&obj->obj,
 								      &object_type,
@@ -682,7 +673,7 @@ static PyObject *DrgnObject_value_impl(struct drgn_object *obj)
 		if (drgn_type_kind(underlying_type) == DRGN_TYPE_BOOL)
 			Py_RETURN_BOOL(uvalue);
 		else
-			return PyLong_FromUint64(uvalue);
+			return PyLong_FromUInt64(uvalue);
 	}
 	case DRGN_OBJECT_ENCODING_SIGNED_BIG:
 	case DRGN_OBJECT_ENCODING_UNSIGNED_BIG: {
@@ -915,10 +906,8 @@ static PyObject *DrgnObject_repr(DrgnObject *self)
 
 static PyObject *DrgnObject_str(DrgnObject *self)
 {
-	struct drgn_error *err;
 	_cleanup_free_ char *str = NULL;
-	err = drgn_format_object(&self->obj, SIZE_MAX,
-				 DRGN_FORMAT_OBJECT_PRETTY, &str);
+	struct drgn_error *err = drgn_format_object(&self->obj, NULL, &str);
 	if (err)
 		return set_drgn_error(err);
 	return PyUnicode_FromString(str);
@@ -969,14 +958,19 @@ static PyObject *DrgnObject_format(DrgnObject *self, PyObject *args,
 		FLAGS
 #undef X
 		"columns",
+		"integer_base",
 		NULL,
 	};
 	struct drgn_error *err;
 	PyObject *columns_obj = Py_None;
-	size_t columns = SIZE_MAX;
-	enum drgn_format_object_flags flags = DRGN_FORMAT_OBJECT_PRETTY;
+	PyObject *integer_base_obj = Py_None;
+	struct drgn_format_object_options options = {
+		.columns = SIZE_MAX,
+		.flags = DRGN_FORMAT_OBJECT_PRETTY,
+		.integer_base = 10,
+	};
 #define X(name, value)	\
-	struct format_object_flag_arg name##_arg = { &flags, value };
+	struct format_object_flag_arg name##_arg = { &options.flags, value };
 	FLAGS
 #undef X
 
@@ -984,25 +978,40 @@ static PyObject *DrgnObject_format(DrgnObject *self, PyObject *args,
 #define X(name, value) "O&"
 					 FLAGS
 #undef X
-					 "O:format_", keywords,
+					 "OO:format_", keywords,
 #define X(name, value) format_object_flag_converter, &name##_arg,
 					 FLAGS
 #undef X
-					 &columns_obj))
+					 &columns_obj, &integer_base_obj))
 		return NULL;
 
 	if (columns_obj != Py_None) {
 		columns_obj = PyNumber_Index(columns_obj);
 		if (!columns_obj)
 			return NULL;
-		columns = PyLong_AsSize_t(columns_obj);
+		options.columns = PyLong_AsSize_t(columns_obj);
 		Py_DECREF(columns_obj);
-		if (columns == (size_t)-1 && PyErr_Occurred())
+		if (options.columns == (size_t)-1 && PyErr_Occurred())
 			return NULL;
 	}
 
+	if (integer_base_obj != Py_None) {
+		int overflow;
+		long integer_base = PyLong_AsLongAndOverflow(integer_base_obj,
+							     &overflow);
+		if (integer_base == -1 && PyErr_Occurred())
+			return NULL;
+		if (overflow
+		    || integer_base < INT_MIN || integer_base > INT_MAX) {
+			PyErr_SetString(PyExc_ValueError,
+					"invalid integer base");
+			return NULL;
+		}
+		options.integer_base = integer_base;
+	}
+
 	_cleanup_free_ char *str = NULL;
-	err = drgn_format_object(&self->obj, columns, flags, &str);
+	err = drgn_format_object(&self->obj, &options, &str);
 	if (err)
 		return set_drgn_error(err);
 	return PyUnicode_FromString(str);
@@ -1037,7 +1046,7 @@ static PyObject *DrgnObject_get_absence_reason(DrgnObject *self, void *arg)
 static PyObject *DrgnObject_get_address(DrgnObject *self, void *arg)
 {
 	if (self->obj.kind == DRGN_OBJECT_REFERENCE)
-		return PyLong_FromUint64(self->obj.address);
+		return PyLong_FromUInt64(self->obj.address);
 	else
 		Py_RETURN_NONE;
 }
@@ -1046,7 +1055,7 @@ static PyObject *DrgnObject_get_bit_offset(DrgnObject *self, void *arg)
 {
 	SWITCH_ENUM(self->obj.kind) {
 	case DRGN_OBJECT_REFERENCE:
-		return PyLong_FromUint8(self->obj.bit_offset);
+		return PyLong_FromUInt8(self->obj.bit_offset);
 	case DRGN_OBJECT_VALUE:
 	case DRGN_OBJECT_ABSENT:
 		Py_RETURN_NONE;
@@ -1058,7 +1067,7 @@ static PyObject *DrgnObject_get_bit_offset(DrgnObject *self, void *arg)
 static PyObject *DrgnObject_get_bit_field_size(DrgnObject *self, void *arg)
 {
 	if (self->obj.is_bit_field)
-		return PyLong_FromUint64(self->obj.bit_size);
+		return PyLong_FromUInt64(self->obj.bit_size);
 	else
 		Py_RETURN_NONE;
 }
@@ -1366,42 +1375,43 @@ static DrgnObject *DrgnObject_member(DrgnObject *self, PyObject *args,
 	return_ptr(res);
 }
 
+static DrgnObject *DrgnObject_subobject(DrgnObject *self, PyObject *args,
+					PyObject *kwds)
+{
+	static char *keywords[] = {"designator", NULL};
+	struct drgn_error *err;
+	const char *designator;
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "s:subobject_", keywords,
+					 &designator))
+		return NULL;
+
+	_cleanup_pydecref_ DrgnObject *res =
+		DrgnObject_alloc(DrgnObject_prog(self));
+	if (!res)
+		return NULL;
+	err = drgn_object_subobject(&res->obj, &self->obj, designator);
+	if (err)
+		return set_drgn_error(err);
+	return_ptr(res);
+}
+
 static PyObject *DrgnObject_getattro(DrgnObject *self, PyObject *attr_name)
 {
 	struct drgn_error *err;
-	PyObject *attr;
-	const char *name;
-	DrgnObject *res;
 
-	/*
-	 * In Python 3.7 and newer, _PyObject_GenericGetAttrWithDict() can
-	 * suppress the AttributeError if the attribute isn't found. This makes
-	 * member lookups much more efficient.
-	 */
-#define GETATTR_SUPPRESS (PY_VERSION_HEX >= 0x030700b1)
-#if GETATTR_SUPPRESS
-	attr = _PyObject_GenericGetAttrWithDict((PyObject *)self, attr_name,
-						NULL, 1);
+	PyObject *attr = _PyObject_GenericGetAttrWithDict((PyObject *)self,
+							  attr_name, NULL, 1);
 	if (attr || PyErr_Occurred())
 		return attr;
-#else
-	PyObject *exc_type, *exc_value, *exc_traceback;
 
-	attr = PyObject_GenericGetAttr((PyObject *)self, attr_name);
-	if (attr || !PyErr_ExceptionMatches(PyExc_AttributeError))
-		return attr;
-	PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
-#endif
+	const char *name = PyUnicode_AsUTF8(attr_name);
+	if (!name)
+		return NULL;
 
-	name = PyUnicode_AsUTF8(attr_name);
-	if (!name) {
-		res = NULL;
-		goto out;
-	}
-
-	res = DrgnObject_alloc(DrgnObject_prog(self));
+	_cleanup_pydecref_ DrgnObject *res =
+		DrgnObject_alloc(DrgnObject_prog(self));
 	if (!res)
-		goto out;
+		return NULL;
 
 	if (self->obj.encoding == DRGN_OBJECT_ENCODING_UNSIGNED) {
 		err = drgn_object_member_dereference(&res->obj, &self->obj,
@@ -1409,38 +1419,20 @@ static PyObject *DrgnObject_getattro(DrgnObject *self, PyObject *attr_name)
 	} else {
 		err = drgn_object_member(&res->obj, &self->obj, name);
 	}
-	if (err) {
-		Py_CLEAR(res);
-		if (err->code == DRGN_ERROR_TYPE) {
-			/*
-			 * If the object doesn't have a compound type, raise a
-			 * generic AttributeError (or restore the original one
-			 * if we weren't able to suppress it).
-			 */
-#if GETATTR_SUPPRESS
-			PyErr_Format(PyExc_AttributeError,
-				     "'%s' object has no attribute '%U'",
-				     Py_TYPE(self)->tp_name, attr_name);
-#else
-			PyErr_Restore(exc_type, exc_value, exc_traceback);
-#endif
-			drgn_error_destroy(err);
-			return NULL;
-		} else if (err->code == DRGN_ERROR_LOOKUP) {
-			PyErr_SetString(PyExc_AttributeError, err->message);
-			drgn_error_destroy(err);
-		} else {
-			set_drgn_error(err);
-		}
+	if (drgn_error_catch(&err, DRGN_ERROR_TYPE)) {
+		// If the object doesn't have a compound type, raise a generic
+		// AttributeError.
+		return PyErr_Format(PyExc_AttributeError,
+				    "'%s' object has no attribute '%U'",
+				    Py_TYPE(self)->tp_name, attr_name);
+	} else if (err && err->code == DRGN_ERROR_LOOKUP) {
+		PyErr_SetString(PyExc_AttributeError, err->message);
+		drgn_error_destroy(err);
+		return NULL;
+	} else if (err) {
+		return set_drgn_error(err);
 	}
-out:
-#if !GETATTR_SUPPRESS
-	Py_XDECREF(exc_traceback);
-	Py_XDECREF(exc_value);
-	Py_DECREF(exc_type);
-#endif
-#undef GETATTR_SUPPRESS
-	return (PyObject *)res;
+	return (PyObject *)no_cleanup_ptr(res);
 }
 
 static Py_ssize_t DrgnObject_length(DrgnObject *self)
@@ -1477,16 +1469,69 @@ static DrgnObject *DrgnObject_subscript_impl(DrgnObject *self,
 	return_ptr(res);
 }
 
-static DrgnObject *DrgnObject_subscript(DrgnObject *self, PyObject *key)
+static DrgnObject *DrgnObject_subscript(DrgnObject *self, PyObject *item)
 {
-	struct index_arg index = { .is_signed = true };
-
-	if (!index_converter(key, &index))
+	if (PyIndex_Check(item)) {
+		int64_t index;
+		if (PyLong_AsInt64(item, &index))
+			return NULL;
+		return DrgnObject_subscript_impl(self, index);
+	} else if (PySlice_Check(item)) {
+		PySliceObject *slice = (PySliceObject *)item;
+		int64_t start, stop;
+		if (slice->start == Py_None)
+			start = 0;
+		else if (PyLong_AsInt64(slice->start, &start))
+			return NULL;
+		if (slice->stop == Py_None) {
+			struct drgn_type *underlying_type =
+				drgn_underlying_type(self->obj.type);
+			if (drgn_type_kind(underlying_type) != DRGN_TYPE_ARRAY
+			    || !drgn_type_is_complete(underlying_type)) {
+				set_error_type_name("'%s' has no length; slice stop must be given",
+						    drgn_object_qualified_type(&self->obj));
+				return NULL;
+			}
+			uint64_t length = drgn_type_length(underlying_type);
+			if (length > INT64_MAX) {
+				PyErr_SetString(PyExc_OverflowError,
+						"length is too large");
+				return NULL;
+			}
+			stop = length;
+		} else if (PyLong_AsInt64(slice->stop, &stop)) {
+			return NULL;
+		}
+		if (slice->step != Py_None) {
+			Py_ssize_t step =
+				PyNumber_AsSsize_t(slice->step,
+						   PyExc_OverflowError);
+			if (step == -1 && PyErr_Occurred())
+				return NULL;
+			if (step != 1) {
+				PyErr_SetString(PyExc_ValueError,
+						"object slice step must be 1");
+				return NULL;
+			}
+		}
+		struct drgn_error *err;
+		_cleanup_pydecref_ DrgnObject *res =
+			DrgnObject_alloc(DrgnObject_prog(self));
+		if (!res)
+			return NULL;
+		err = drgn_object_slice(&res->obj, &self->obj, start, stop);
+		if (err)
+			return set_drgn_error(err);
+		return_ptr(res);
+	} else {
+		PyErr_Format(PyExc_TypeError,
+			     "object subscript must be integer or slice, not %.200s",
+			     Py_TYPE(item)->tp_name);
 		return NULL;
-	return DrgnObject_subscript_impl(self, index.svalue);
+	}
 }
 
-static ObjectIterator *DrgnObject_iter(DrgnObject *self)
+static ObjectIterator *DrgnObject_iter_impl(DrgnObject *self, bool reversed)
 {
 	struct drgn_type *underlying_type =
 		drgn_underlying_type(self->obj.type);
@@ -1502,10 +1547,28 @@ static ObjectIterator *DrgnObject_iter(DrgnObject *self)
 		return NULL;
 	it->obj = self;
 	Py_INCREF(self);
-	it->length = drgn_type_length(underlying_type);
+	if (reversed) {
+		it->index = drgn_type_length(underlying_type) - 1;
+		it->end = -1;
+		it->step = -1;
+	} else {
+		it->index = 0;
+		it->end = drgn_type_length(underlying_type);
+		it->step = 1;
+	}
 	return it;
 }
 
+static ObjectIterator *DrgnObject_iter(DrgnObject *self)
+{
+	return DrgnObject_iter_impl(self, false);
+}
+
+static ObjectIterator *DrgnObject_reversed(DrgnObject *self)
+{
+	return DrgnObject_iter_impl(self, true);
+}
+
 static int add_to_dir(PyObject *dir, struct drgn_type *type)
 {
 	struct drgn_error *err;
@@ -1588,6 +1651,8 @@ static PyMethodDef DrgnObject_methods[] = {
 	 drgn_Object_string__DOC},
 	{"member_", (PyCFunction)DrgnObject_member,
 	 METH_VARARGS | METH_KEYWORDS, drgn_Object_member__DOC},
+	{"subobject_", (PyCFunction)DrgnObject_subobject,
+	 METH_VARARGS | METH_KEYWORDS, drgn_Object_subobject__DOC},
 	{"address_of_", (PyCFunction)DrgnObject_address_of, METH_NOARGS,
 	 drgn_Object_address_of__DOC},
 	{"read_", (PyCFunction)DrgnObject_read, METH_NOARGS,
@@ -1599,6 +1664,7 @@ static PyMethodDef DrgnObject_methods[] = {
 	 drgn_Object_from_bytes__DOC},
 	{"format_", (PyCFunction)DrgnObject_format,
 	 METH_VARARGS | METH_KEYWORDS, drgn_Object_format__DOC},
+	{"__reversed__", (PyCFunction)DrgnObject_reversed, METH_NOARGS},
 	{"__round__", (PyCFunction)DrgnObject_round,
 	 METH_VARARGS | METH_KEYWORDS},
 	{"__trunc__", (PyCFunction)DrgnObject_trunc, METH_NOARGS},
@@ -1701,7 +1767,43 @@ DrgnObject *op(PyObject *self, PyObject *args, PyObject *kwds)			\
 }
 
 DrgnObject_CAST_OP(cast)
-DrgnObject_CAST_OP(implicit_convert)
+
+DrgnObject *implicit_convert(PyObject *self, PyObject *args, PyObject *kwds)
+{
+	static char *keywords[] = {"type", "obj", "bit_field_size", NULL};
+	struct drgn_error *err;
+	PyObject *type_obj;
+	DrgnObject *obj;
+	struct index_arg bit_field_size = { .allow_none = true, .is_none = true };
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO!|$O&:implicit_convert",
+					 keywords, &type_obj, &DrgnObject_type,
+					 &obj, index_converter,
+					 &bit_field_size))
+		return NULL;
+
+	if (!bit_field_size.is_none && bit_field_size.uvalue == 0) {
+		PyErr_SetString(PyExc_ValueError,
+				"bit field size cannot be zero");
+		return NULL;
+	}
+
+	struct drgn_qualified_type qualified_type;
+	if (Program_type_arg(DrgnObject_prog(obj), type_obj, false,
+			     &qualified_type) == -1)
+		return NULL;
+
+	_cleanup_pydecref_ DrgnObject *res =
+		DrgnObject_alloc(DrgnObject_prog(obj));
+	if (!res)
+		return NULL;
+
+	err = drgn_object_implicit_convert(&res->obj, qualified_type,
+					   bit_field_size.uvalue, &obj->obj);
+	if (err)
+		return set_drgn_error(err);
+	return_ptr(res);
+}
+
 DrgnObject_CAST_OP(reinterpret)
 
 #undef DrgnObject_CAST_OP
@@ -1751,14 +1853,22 @@ static int ObjectIterator_traverse(ObjectIterator *self, visitproc visit,
 
 static DrgnObject *ObjectIterator_next(ObjectIterator *self)
 {
-	if (self->index >= self->length)
+	if (self->index == self->end)
 		return NULL;
-	return DrgnObject_subscript_impl(self->obj, self->index++);
+	DrgnObject *ret = DrgnObject_subscript_impl(self->obj, self->index);
+	if (ret)
+		self->index += self->step;
+	return ret;
 }
 
 static PyObject *ObjectIterator_length_hint(ObjectIterator *self)
 {
-	return PyLong_FromUint64(self->length);
+	uint64_t length;
+	if (self->step == 1)
+		length = self->end - self->index;
+	else
+		length = self->index + 1;
+	return PyLong_FromUInt64(length);
 }
 
 static PyMethodDef ObjectIterator_methods[] = {
