@@ -146,14 +146,10 @@ static int get_logging_status(int *log_level_ret, bool *enable_progress_bar_ret)
 // and there are no APIs to be notified of this.
 //
 // To sync the log level, we monkey patch logger._cache.clear() to update the
-// libdrgn log level on every live program. This only works since CPython commit
-// 78c18a9b9a14 ("bpo-30962: Added caching to Logger.isEnabledFor() (GH-2752)")
-// (in v3.7), though. Before that, the best we can do is sync the level at the
-// time that the program is created.
+// libdrgn log level on every live program.
 //
 // We also check handlers in that monkey patch, which isn't the right place to
 // hook but should work in practice in most cases.
-#if PY_VERSION_HEX >= 0x030700a1
 static int cached_log_level;
 static bool cached_enable_progress_bar;
 static struct pyobjectp_set programs = HASH_TABLE_INIT;
@@ -228,24 +224,6 @@ static void Program_deinit_logging(Program *prog)
 	PyObject *obj = (PyObject *)prog;
 	pyobjectp_set_delete(&programs, &obj);
 }
-#else
-static int init_logger_cache_wrapper(void) { return 0; }
-
-static int Program_init_logging(Program *prog)
-{
-	int level;
-	bool enable_progress_bar;
-	if (get_logging_status(&level, &enable_progress_bar))
-		return -1;
-	drgn_program_set_log_callback(&prog->prog, drgnpy_log_fn, NULL);
-	drgn_program_set_log_level(&prog->prog, level);
-	drgn_program_set_progress_file(&prog->prog,
-				       enable_progress_bar ? stderr : NULL);
-	return 0;
-}
-
-static void Program_deinit_logging(Program *prog) {}
-#endif
 
 int init_logging(void)
 {
@@ -329,18 +307,22 @@ int Program_type_arg(Program *prog, PyObject *type_obj, bool can_be_none,
 	return 0;
 }
 
-void *drgn_begin_blocking(void)
+drgn_blocking_state drgn_begin_blocking(void)
 {
-	PyThreadState *state = PyThreadState_GetUnchecked();
-	if (state)
-		PyEval_ReleaseThread(state);
-	return state;
+	PyThreadState *state = PyGILState_GetThisThreadState();
+	// If the current thread doesn't hold the GIL (because it's not in
+	// Python or because this blocking section is nested in another), don't
+	// do anything.
+	if (!state || state != PyThreadState_GetUnchecked())
+		return NULL;
+	PyEval_ReleaseThread(state);
+	return (drgn_blocking_state)state;
 }
 
-void drgn_end_blocking(void *state)
+void drgn_end_blocking(drgn_blocking_state state)
 {
 	if (state)
-		PyEval_RestoreThread(state);
+		PyEval_RestoreThread((PyThreadState *)state);
 }
 
 static Program *Program_new_impl(const struct drgn_platform *platform)
@@ -348,11 +330,15 @@ static Program *Program_new_impl(const struct drgn_platform *platform)
 	_cleanup_pydecref_ PyObject *cache = PyDict_New();
 	if (!cache)
 		return NULL;
+	_cleanup_pydecref_ PyObject *config = PyDict_New();
+	if (!config)
+		return NULL;
 
 	_cleanup_pydecref_ Program *prog = call_tp_alloc(Program);
 	if (!prog)
 		return NULL;
 	prog->cache = no_cleanup_ptr(cache);
+	prog->config = no_cleanup_ptr(config);
 	pyobjectp_set_init(&prog->objects);
 	drgn_program_init(&prog->prog, platform);
 	if (Program_init_logging(prog))
@@ -425,6 +411,7 @@ static void Program_dealloc(Program *self)
 	hash_table_for_each(pyobjectp_set, it, &self->objects)
 		Py_DECREF(*it.entry);
 	pyobjectp_set_deinit(&self->objects);
+	Py_XDECREF(self->config);
 	Py_XDECREF(self->cache);
 	Py_TYPE(self)->tp_free((PyObject *)self);
 }
@@ -434,6 +421,7 @@ static int Program_traverse(Program *self, visitproc visit, void *arg)
 	hash_table_for_each(pyobjectp_set, it, &self->objects)
 		Py_VISIT(*it.entry);
 	Py_VISIT(self->cache);
+	Py_VISIT(self->config);
 	return 0;
 }
 
@@ -444,6 +432,7 @@ static int Program_clear(Program *self)
 	pyobjectp_set_deinit(&self->objects);
 	pyobjectp_set_init(&self->objects);
 	Py_CLEAR(self->cache);
+	Py_CLEAR(self->config);
 	return 0;
 }
 
@@ -997,6 +986,28 @@ static PyObject *Program_set_kernel(Program *self)
 	Py_RETURN_NONE;
 }
 
+static PyObject *Program_set_linux_kernel_custom(Program *self, PyObject *args,
+						  PyObject *kwds)
+{
+	static char *keywords[] = {"vmcoreinfo", "is_live", NULL};
+	struct drgn_error *err;
+	const char *vmcoreinfo;
+	Py_ssize_t vmcoreinfo_size;
+	int is_live;
+
+	if (!PyArg_ParseTupleAndKeywords(args, kwds,
+					 "s#p:set_linux_kernel_custom", keywords,
+					 &vmcoreinfo, &vmcoreinfo_size,
+					 &is_live))
+		return NULL;
+
+	err = drgn_program_set_linux_kernel_custom(&self->prog, vmcoreinfo,
+						   vmcoreinfo_size, is_live);
+	if (err)
+		return set_drgn_error(err);
+	Py_RETURN_NONE;
+}
+
 static PyObject *Program_set_pid(Program *self, PyObject *args, PyObject *kwds)
 {
 	static char *keywords[] = {"pid", NULL};
@@ -1471,6 +1482,34 @@ static PyObject *Program_read(Program *self, PyObject *args, PyObject *kwds)
 	return_ptr(buf);
 }
 
+static PyObject *Program_read_c_string(Program *self, PyObject *args,
+				       PyObject *kwds)
+{
+	static char *keywords[] = {"address", "physical", "max_size", NULL};
+	struct drgn_error *err;
+	struct index_arg address = {};
+	int physical = 0;
+	Py_ssize_t max_size = PY_SSIZE_T_MAX;
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O&|p$n:read_c_string",
+					 keywords, index_converter, &address,
+					 &physical, &max_size))
+	    return NULL;
+
+	if (max_size < 0) {
+		PyErr_SetString(PyExc_ValueError, "negative max_size");
+		return NULL;
+	}
+	bool clear = set_drgn_in_python();
+	_cleanup_free_ char *str = NULL;
+	err = drgn_program_read_c_string(&self->prog, address.uvalue, physical,
+					 max_size, &str);
+	if (clear)
+		clear_drgn_in_python();
+	if (err)
+		return set_drgn_error(err);
+	return PyBytes_FromString(str);
+}
+
 #define METHOD_READ(x, type)							\
 static PyObject *Program_read_##x(Program *self, PyObject *args,		\
 				  PyObject *kwds)				\
@@ -1501,6 +1540,215 @@ METHOD_READ(u64, uint64_t)
 METHOD_READ(word, uint64_t)
 #undef METHOD_READ
 
+static PyObject *Program_search_memory(Program *self, PyObject *args,
+				       PyObject *kwds)
+{
+	struct drgn_error *err;
+	static char *keywords[] = {"value", "alignment", NULL};
+	PyObject *value_obj;
+	PyObject *alignment_obj = Py_None;
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|$O:search_memory",
+					 keywords, &value_obj, &alignment_obj))
+		return NULL;
+
+	struct drgn_memory_search_iterator *it;
+	if (PyObject_TypeCheck(value_obj, &PyBytes_Type)) {
+		char *needle;
+		Py_ssize_t size;
+		if (PyBytes_AsStringAndSize(value_obj, &needle, &size))
+			return NULL;
+
+		uint64_t alignment;
+		if (alignment_obj == Py_None)
+			alignment = 1;
+		else if (PyLong_AsUInt64(alignment_obj, &alignment))
+			return NULL;
+
+		err = drgn_program_search_memory(&self->prog, needle, size,
+						 alignment, &it);
+		if (err)
+			return set_drgn_error(err);
+	} else if (PyObject_TypeCheck(value_obj, &PyUnicode_Type)) {
+		Py_ssize_t size;
+		const char *needle = PyUnicode_AsUTF8AndSize(value_obj, &size);
+		if (!needle)
+			return NULL;
+
+		uint64_t alignment;
+		if (alignment_obj == Py_None)
+			alignment = 1;
+		else if (PyLong_AsUInt64(alignment_obj, &alignment))
+			return NULL;
+
+		err = drgn_program_search_memory(&self->prog, needle, size,
+						 alignment, &it);
+		if (err)
+			return set_drgn_error(err);
+	} else {
+		if (alignment_obj != Py_None) {
+			PyErr_SetString(PyExc_TypeError,
+					"alignment is only allowed with bytes or str value");
+			return NULL;
+		}
+		if (PyObject_TypeCheck(value_obj, &DrgnObject_type)) {
+			err = drgn_search_memory_for_object(&((DrgnObject *)value_obj)->obj,
+							    &it);
+			if (err)
+				return set_drgn_error(err);
+		} else if (PyIndex_Check(value_obj)) {
+			uint64_t value;
+			if (PyLong_AsUInt64(value_obj, &value))
+				return NULL;
+			err = drgn_program_search_memory_word(&self->prog,
+							      value, &it);
+			if (err)
+				return set_drgn_error(err);
+		} else {
+			return PyErr_Format(PyExc_TypeError,
+					    "value must be bytes, str, int, "
+					    "or drgn.Object, not %.200s",
+					    Py_TYPE(value_obj)->tp_name);
+		}
+	}
+
+	PyObject *ret =
+		MemorySearchIterator_wrap(&MemorySearchIterator_type, it);
+	if (!ret)
+		drgn_memory_search_iterator_destroy(it);
+	return ret;
+}
+
+#define X(bits)									\
+DEFINE_VECTOR(uint##bits##_vector, uint##bits##_t);				\
+DEFINE_VECTOR(uint##bits##_range_vector, uint##bits##_t [2]);			\
+static PyObject *Program_search_memory_u##bits(Program *self, PyObject *args,	\
+					       PyObject *kwds)			\
+{										\
+	struct drgn_error *err;							\
+										\
+	static char *keywords[] = {"ignore_mask", NULL};			\
+	_cleanup_pydecref_ PyObject *empty_tuple = PyTuple_New(0);		\
+	if (!empty_tuple)							\
+		return NULL;							\
+	PyObject *ignore_mask_obj = NULL;					\
+	if (!PyArg_ParseTupleAndKeywords(empty_tuple, kwds,			\
+					 "|$O:search_memory_u" #bits,		\
+					 keywords, &ignore_mask_obj))		\
+		return NULL;							\
+										\
+	VECTOR(uint##bits##_vector, values);					\
+	VECTOR(uint##bits##_range_vector, ranges);				\
+										\
+	Py_ssize_t num_args = PyTuple_GET_SIZE(args);				\
+	for (Py_ssize_t i = 0; i < num_args; i++) {				\
+		PyObject *item = PyTuple_GET_ITEM(args, i);			\
+		if (PyTuple_Check(item)) {					\
+			if (PyTuple_GET_SIZE(item) != 2) {			\
+				PyErr_SetString(PyExc_TypeError,		\
+						"values must be int or (int, int)");\
+				return NULL;					\
+			}							\
+			uint##bits##_t (*entry)[2] =				\
+				uint##bits##_range_vector_append_entry(&ranges);\
+			if (!entry)						\
+				return PyErr_NoMemory();			\
+			if (PyLong_AsUInt##bits(PyTuple_GET_ITEM(item, 0),	\
+						&(*entry)[0])			\
+			    || PyLong_AsUInt##bits(PyTuple_GET_ITEM(item, 1),	\
+						   &(*entry)[1]))		\
+				return NULL;					\
+		} else {							\
+			uint##bits##_t *entry =					\
+				uint##bits##_vector_append_entry(&values);	\
+			if (!entry)						\
+				return PyErr_NoMemory();			\
+			if (PyLong_AsUInt##bits(item, entry))			\
+				return NULL;					\
+		}								\
+	}									\
+										\
+	uint##bits##_t ignore_mask;						\
+	if (!ignore_mask_obj)							\
+		ignore_mask = 0;						\
+	else if (PyLong_AsUInt##bits(ignore_mask_obj, &ignore_mask))		\
+		return NULL;							\
+										\
+	struct drgn_memory_search_iterator *it;					\
+	err = drgn_program_search_memory_u##bits##_multi(&self->prog,		\
+				uint##bits##_vector_begin(&values),		\
+				uint##bits##_vector_size(&values),		\
+				ignore_mask,					\
+				uint##bits##_range_vector_begin(&ranges),	\
+				uint##bits##_range_vector_size(&ranges), &it);	\
+	if (err)								\
+		return set_drgn_error(err);					\
+	PyObject *ret =								\
+		MemorySearchIterator_wrap(&MemorySearchIteratorWithInt_type,	\
+					  it);					\
+	if (!ret)								\
+		drgn_memory_search_iterator_destroy(it);			\
+	return ret;								\
+}
+SEARCH_MEMORY_UINT_SIZES
+#undef X
+
+static PyObject *Program_search_memory_word(Program *self, PyObject *args,
+					    PyObject *kwds)
+{
+	struct drgn_error *err;
+	bool is_64_bit;
+	err = drgn_program_is_64_bit(&self->prog, &is_64_bit);
+	if (err)
+		return set_drgn_error(err);
+	return (is_64_bit
+		? Program_search_memory_u64
+		: Program_search_memory_u32)(self, args, kwds);
+}
+
+static PyObject *Program_search_memory_regex(Program *self, PyObject *args,
+					     PyObject *kwds)
+{
+	struct drgn_error *err;
+	static char *keywords[] = {"pattern", NULL};
+	PyObject *pattern_obj;
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|$:search_memory_regex",
+					 keywords, &pattern_obj))
+		return NULL;
+
+	const char *pattern;
+	Py_ssize_t pattern_len;
+	bool utf8;
+	if (PyObject_TypeCheck(pattern_obj, &PyBytes_Type)) {
+		if (PyBytes_AsStringAndSize(pattern_obj, (char **)&pattern,
+					    &pattern_len))
+			return NULL;
+		utf8 = false;
+	} else if (PyObject_TypeCheck(pattern_obj, &PyUnicode_Type)) {
+		pattern = PyUnicode_AsUTF8AndSize(pattern_obj, &pattern_len);
+		if (!pattern)
+			return NULL;
+		utf8 = true;
+	} else {
+		PyErr_SetString(PyExc_TypeError,
+				"pattern must be bytes or str");
+		return NULL;
+	}
+
+	struct drgn_memory_search_iterator *it;
+	err = drgn_program_search_memory_regex(&self->prog, pattern,
+					       pattern_len, utf8, &it);
+	if (err)
+		return set_drgn_error(err);
+	PyObject *ret =
+		MemorySearchIterator_wrap(utf8
+					  ? &MemorySearchIteratorWithStr_type
+					  : &MemorySearchIteratorWithBytes_type,
+					  it);
+	if (!ret)
+		drgn_memory_search_iterator_destroy(it);
+	return ret;
+}
+
 static PyObject *Program_find_type(Program *self, PyObject *args, PyObject *kwds)
 {
 	static char *keywords[] = {"name", "filename", NULL};
@@ -1542,24 +1790,53 @@ static PyObject *Program_find_type(Program *self, PyObject *args, PyObject *kwds
 	return DrgnType_wrap(qualified_type);
 }
 
-static DrgnObject *Program_find_object(Program *self, const char *name,
-				       struct path_arg *filename,
+static void *set_object_not_found_error(struct drgn_error *err, PyObject *name)
+{
+	_cleanup_pydecref_ PyObject *args = Py_BuildValue("(s)", err->message);
+	drgn_error_destroy(err);
+	if (!args)
+		return NULL;
+
+	_cleanup_pydecref_ PyObject *kwargs =
+		Py_BuildValue("{sO}", "name", name);
+	if (!kwargs)
+		return NULL;
+
+	_cleanup_pydecref_ PyObject *exc =
+		PyObject_Call((PyObject *)&ObjectNotFoundError_type, args,
+			      kwargs);
+	if (exc)
+		PyErr_SetObject((PyObject *)&ObjectNotFoundError_type, exc);
+	return NULL;
+}
+
+static DrgnObject *Program_find_object(Program *self, PyObject *name_obj,
+				       const char *filename,
 				       enum drgn_find_object_flags flags)
 {
 	struct drgn_error *err;
 
+	if (!PyUnicode_Check(name_obj)) {
+		PyErr_Format(PyExc_TypeError, "name must be str, not %.200s",
+			     Py_TYPE(name_obj)->tp_name);
+		return NULL;
+	}
+	const char *name = PyUnicode_AsUTF8(name_obj);
+	if (!name)
+		return NULL;
+
 	_cleanup_pydecref_ DrgnObject *ret = DrgnObject_alloc(self);
 	if (!ret)
 		return NULL;
 	bool clear = set_drgn_in_python();
-	err = drgn_program_find_object(&self->prog, name, filename->path, flags,
+	err = drgn_program_find_object(&self->prog, name, filename, flags,
 				       &ret->obj);
 	if (clear)
 		clear_drgn_in_python();
-	if (err) {
-		set_drgn_error(err);
-		return NULL;
-	}
+	if (err && err->code == DRGN_ERROR_LOOKUP)
+		return set_object_not_found_error(err, name_obj);
+	else if (err)
+		return set_drgn_error(err);
 	return_ptr(ret);
 }
 
@@ -1567,33 +1844,31 @@ static DrgnObject *Program_object(Program *self, PyObject *args,
 				  PyObject *kwds)
 {
 	static char *keywords[] = {"name", "flags", "filename", NULL};
-	const char *name;
+	PyObject *name;
 	struct enum_arg flags = {
 		.type = FindObjectFlags_class,
 		.value = DRGN_FIND_OBJECT_ANY,
 	};
 	PATH_ARG(filename, .allow_none = true);
-
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "s|O&O&:object", keywords,
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O&O&:object", keywords,
 					 &name, enum_converter, &flags,
 					 path_converter, &filename))
 		return NULL;
 
-	return Program_find_object(self, name, &filename, flags.value);
+	return Program_find_object(self, name, filename.path, flags.value);
 }
 
 static DrgnObject *Program_constant(Program *self, PyObject *args,
 				    PyObject *kwds)
 {
 	static char *keywords[] = {"name", "filename", NULL};
-	const char *name;
+	PyObject *name;
 	PATH_ARG(filename, .allow_none = true);
-
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "s|O&:constant", keywords,
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O&:constant", keywords,
 					 &name, path_converter, &filename))
 		return NULL;
 
-	return Program_find_object(self, name, &filename,
+	return Program_find_object(self, name, filename.path,
 				   DRGN_FIND_OBJECT_CONSTANT);
 }
 
@@ -1601,14 +1876,13 @@ static DrgnObject *Program_function(Program *self, PyObject *args,
 				    PyObject *kwds)
 {
 	static char *keywords[] = {"name", "filename", NULL};
-	const char *name;
+	PyObject *name;
 	PATH_ARG(filename, .allow_none = true);
-
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "s|O&:function", keywords,
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O&:function", keywords,
 					 &name, path_converter, &filename))
 		return NULL;
 
-	return Program_find_object(self, name, &filename,
+	return Program_find_object(self, name, filename.path,
 				   DRGN_FIND_OBJECT_FUNCTION);
 }
 
@@ -1616,14 +1890,13 @@ static DrgnObject *Program_variable(Program *self, PyObject *args,
 				    PyObject *kwds)
 {
 	static char *keywords[] = {"name", "filename", NULL};
-	const char *name;
+	PyObject *name;
 	PATH_ARG(filename, .allow_none = true);
-
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "s|O&:variable", keywords,
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O&:variable", keywords,
 					 &name, path_converter, &filename))
 		return NULL;
 
-	return Program_find_object(self, name, &filename,
+	return Program_find_object(self, name, filename.path,
 				   DRGN_FIND_OBJECT_VARIABLE);
 }
 
@@ -1695,6 +1968,32 @@ static PyObject *Program_stack_trace_from_pcs(Program *self, PyObject *args,
 	return ret;
 }
 
+static PyObject *Program_source_location(Program *self, PyObject *arg)
+{
+	struct drgn_error *err;
+
+	struct drgn_source_location_list *locs;
+	if (PyUnicode_Check(arg)) {
+		const char *address_str = PyUnicode_AsUTF8(arg);
+		if (!address_str)
+			return NULL;
+		err = drgn_program_addr2line(&self->prog, address_str, &locs);
+	} else {
+		struct index_arg address = {};
+		if (!index_converter(arg, &address))
+			return NULL;
+		err = drgn_program_source_location(&self->prog, address.uvalue,
+						   &locs);
+	}
+	if (err)
+		return set_drgn_error(err);
+
+	PyObject *ret = SourceLocationList_wrap(locs);
+	if (!ret)
+		drgn_source_location_list_destroy(locs);
+	return ret;
+}
+
 static PyObject *Program_symbols(Program *self, PyObject *args)
 {
 	struct drgn_error *err;
@@ -1820,6 +2119,16 @@ static PyObject *Program_crashed_thread(Program *self)
 	return Thread_wrap(thread);
 }
 
+static PyObject *Program_address_size(Program *self)
+{
+	struct drgn_error *err;
+	uint64_t address_size;
+	err = drgn_program_address_size(&self->prog, &address_size);
+	if (err)
+		return set_drgn_error(err);
+	return PyLong_FromUInt64(address_size);
+}
+
 // Used for testing.
 static PyObject *Program__log(Program *self, PyObject *args, PyObject *kwds)
 {
@@ -1833,36 +2142,11 @@ static PyObject *Program__log(Program *self, PyObject *args, PyObject *kwds)
 
 static DrgnObject *Program_subscript(Program *self, PyObject *key)
 {
-	struct drgn_error *err;
-
 	if (!PyUnicode_Check(key)) {
 		PyErr_SetObject(PyExc_KeyError, key);
 		return NULL;
 	}
-
-	const char *name = PyUnicode_AsUTF8(key);
-	if (!name)
-		return NULL;
-
-	_cleanup_pydecref_ DrgnObject *ret = DrgnObject_alloc(self);
-	if (!ret)
-		return NULL;
-
-	bool clear = set_drgn_in_python();
-	err = drgn_program_find_object(&self->prog, name, NULL,
-				       DRGN_FIND_OBJECT_ANY, &ret->obj);
-	if (clear)
-		clear_drgn_in_python();
-	if (err) {
-		if (err->code == DRGN_ERROR_LOOKUP) {
-			drgn_error_destroy(err);
-			PyErr_SetObject(PyExc_KeyError, key);
-		} else {
-			set_drgn_error(err);
-		}
-		return NULL;
-	}
-	return_ptr(ret);
+	return Program_find_object(self, key, NULL, DRGN_FIND_OBJECT_ANY);
 }
 
 static int Program_contains(Program *self, PyObject *key)
@@ -1913,6 +2197,14 @@ static PyObject *Program_get_platform(Program *self, void *arg)
 		Py_RETURN_NONE;
 }
 
+static PyObject *Program_get_core_dump_path(Program *self, void *arg)
+{
+	const char *path = drgn_program_core_dump_path(&self->prog);
+	if (!path)
+		Py_RETURN_NONE;
+	return PyUnicode_FromString(path);
+}
+
 static PyObject *Program_get_language(Program *self, void *arg)
 {
 	return Language_wrap(drgn_program_language(&self->prog));
@@ -1960,6 +2252,8 @@ static PyMethodDef Program_methods[] = {
 	 METH_VARARGS | METH_KEYWORDS, drgn_Program_set_core_dump_DOC},
 	{"set_kernel", (PyCFunction)Program_set_kernel, METH_NOARGS,
 	 drgn_Program_set_kernel_DOC},
+	{"set_linux_kernel_custom", (PyCFunction)Program_set_linux_kernel_custom,
+	 METH_VARARGS | METH_KEYWORDS, drgn_Program_set_linux_kernel_custom_DOC},
 	{"set_pid", (PyCFunction)Program_set_pid, METH_VARARGS | METH_KEYWORDS,
 	 drgn_Program_set_pid_DOC},
 	{"modules", (PyCFunction)Program_modules, METH_NOARGS,
@@ -2001,6 +2295,8 @@ static PyMethodDef Program_methods[] = {
 	 drgn_Program___contains___DOC},
 	{"read", (PyCFunction)Program_read, METH_VARARGS | METH_KEYWORDS,
 	 drgn_Program_read_DOC},
+	{"read_c_string", (PyCFunction)Program_read_c_string,
+	 METH_VARARGS | METH_KEYWORDS, drgn_Program_read_c_string_DOC},
 #define METHOD_DEF_READ(x)						\
 	{"read_"#x, (PyCFunction)Program_read_##x,			\
 	 METH_VARARGS | METH_KEYWORDS, drgn_Program_read_##x##_DOC}
@@ -2010,6 +2306,18 @@ static PyMethodDef Program_methods[] = {
 	METHOD_DEF_READ(u64),
 	METHOD_DEF_READ(word),
 #undef METHOD_READ_U
+	{"search_memory", (PyCFunction)Program_search_memory,
+	 METH_VARARGS | METH_KEYWORDS, drgn_Program_search_memory_DOC},
+#define X(bits)									\
+	{"search_memory_u" #bits, (PyCFunction)Program_search_memory_u##bits,	\
+	 METH_VARARGS | METH_KEYWORDS,						\
+	 drgn_Program_search_memory_u##bits##_DOC},
+	SEARCH_MEMORY_UINT_SIZES
+#undef X
+	{"search_memory_word", (PyCFunction)Program_search_memory_word,
+	 METH_VARARGS | METH_KEYWORDS, drgn_Program_search_memory_word_DOC},
+	{"search_memory_regex", (PyCFunction)Program_search_memory_regex,
+	 METH_VARARGS | METH_KEYWORDS, drgn_Program_search_memory_regex_DOC},
 	{"type", (PyCFunction)Program_find_type, METH_VARARGS | METH_KEYWORDS,
 	 drgn_Program_type_DOC},
 	{"object", (PyCFunction)Program_object, METH_VARARGS | METH_KEYWORDS,
@@ -2024,6 +2332,8 @@ static PyMethodDef Program_methods[] = {
 	 METH_VARARGS | METH_KEYWORDS, drgn_Program_stack_trace_DOC},
 	{"stack_trace_from_pcs", (PyCFunction)Program_stack_trace_from_pcs,
 	 METH_VARARGS | METH_KEYWORDS, drgn_Program_stack_trace_from_pcs_DOC},
+	{"source_location", (PyCFunction)Program_source_location, METH_O,
+	 drgn_Program_source_location_DOC},
 	{"symbols", (PyCFunction)Program_symbols, METH_VARARGS,
 	 drgn_Program_symbols_DOC},
 	{"symbol", (PyCFunction)Program_symbol, METH_O,
@@ -2036,6 +2346,8 @@ static PyMethodDef Program_methods[] = {
 	 drgn_Program_main_thread_DOC},
 	{"crashed_thread", (PyCFunction)Program_crashed_thread, METH_NOARGS,
 	 drgn_Program_crashed_thread_DOC},
+	{"address_size", (PyCFunction)Program_address_size, METH_NOARGS,
+	 drgn_Program_address_size_DOC},
 	{"void_type", (PyCFunction)Program_void_type,
 	 METH_VARARGS | METH_KEYWORDS, drgn_Program_void_type_DOC},
 	{"int_type", (PyCFunction)Program_int_type,
@@ -2067,6 +2379,8 @@ static PyMethodDef Program_methods[] = {
 static PyMemberDef Program_members[] = {
 	{"cache", T_OBJECT_EX, offsetof(Program, cache), 0,
 	 drgn_Program_cache_DOC},
+	{"config", T_OBJECT_EX, offsetof(Program, config), 0,
+	 drgn_Program_config_DOC},
 	{},
 };
 
@@ -2074,6 +2388,8 @@ static PyGetSetDef Program_getset[] = {
 	{"flags", (getter)Program_get_flags, NULL, drgn_Program_flags_DOC},
 	{"platform", (getter)Program_get_platform, NULL,
 	 drgn_Program_platform_DOC},
+	{"core_dump_path", (getter)Program_get_core_dump_path, NULL,
+	 drgn_Program_core_dump_path_DOC},
 	{"language", (getter)Program_get_language, (setter)Program_set_language,
 	 drgn_Program_language_DOC},
 	{"debug_info_options", (getter)Program_get_debug_info_options,
