@@ -16,6 +16,18 @@
 #include "type.h"
 #include "util.h"
 
+LIBDRGN_PUBLIC struct drgn_qualified_type
+drgn_qualified_type_unaliased(struct drgn_qualified_type qualified_type)
+{
+	while (drgn_type_kind(qualified_type.type) == DRGN_TYPE_TYPEDEF) {
+		struct drgn_qualified_type tmp =
+			drgn_type_type(qualified_type.type);
+		qualified_type.qualifiers |= tmp.qualifiers;
+		qualified_type.type = tmp.type;
+	}
+	return qualified_type;
+}
+
 const char * const drgn_type_kind_spelling[] = {
 	[DRGN_TYPE_VOID] = "void",
 	[DRGN_TYPE_INT] = "int",
@@ -1044,6 +1056,14 @@ drgn_format_type(struct drgn_qualified_type qualified_type, char **ret)
 	return lang->format_type(qualified_type, ret);
 }
 
+LIBDRGN_PUBLIC struct drgn_error *
+drgn_format_variable_declaration(struct drgn_qualified_type qualified_type,
+				 const char *name, char **ret)
+{
+	const struct drgn_language *lang = drgn_type_language(qualified_type.type);
+	return lang->format_variable_declaration(qualified_type, name, ret);
+}
+
 bool drgn_type_is_integer(struct drgn_type *type)
 {
 	switch (drgn_type_kind(type)) {
@@ -1434,7 +1454,7 @@ default_size_t_or_ptrdiff_t(struct drgn_program *prog,
 		},
 	};
 	struct drgn_error *err;
-	uint8_t address_size;
+	uint64_t address_size;
 	err = drgn_program_address_size(prog, &address_size);
 	if (err)
 		return err;
@@ -1529,7 +1549,7 @@ drgn_program_find_primitive_type(struct drgn_program *prog,
 	/* long and unsigned long default to the word size. */
 	case DRGN_C_TYPE_LONG:
 	case DRGN_C_TYPE_UNSIGNED_LONG: {
-		uint8_t address_size;
+		uint64_t address_size;
 		err = drgn_program_address_size(prog, &address_size);
 		if (err)
 			break;
@@ -1640,8 +1660,8 @@ drgn_type_offsetof(struct drgn_type *type, const char *member_designator,
 	struct drgn_error *err;
 	const struct drgn_language *lang = drgn_type_language(type);
 	uint64_t bit_offset;
-	err = lang->bit_offset(drgn_type_program(type), type, member_designator,
-			       &bit_offset);
+	err = lang->type_subobject(type, member_designator, true, NULL,
+				   &bit_offset, NULL);
 	if (err)
 		return err;
 	if (bit_offset % 8) {
