@@ -16,6 +16,7 @@ from drgn import (
     Architecture,
     FindObjectFlags,
     Language,
+    NoDefaultProgramError,
     Object,
     Platform,
     PlatformFlags,
@@ -27,6 +28,8 @@ from drgn import (
     TypeMember,
     TypeParameter,
     TypeTemplateParameter,
+    get_default_prog,
+    set_default_prog,
 )
 
 try:
@@ -43,10 +46,20 @@ else:
 DEFAULT_LANGUAGE = Language.C
 
 
+class IntWrapper:
+    def __init__(self, value):
+        self._value = value
+
+    def __index__(self):
+        return self._value
+
+
 MOCK_32BIT_PLATFORM = Platform(Architecture.UNKNOWN, PlatformFlags.IS_LITTLE_ENDIAN)
+MOCK_32BIT_BIG_ENDIAN_PLATFORM = Platform(Architecture.UNKNOWN, PlatformFlags(0))
 MOCK_PLATFORM = Platform(
     Architecture.UNKNOWN, PlatformFlags.IS_64_BIT | PlatformFlags.IS_LITTLE_ENDIAN
 )
+MOCK_BIG_ENDIAN_PLATFORM = Platform(Architecture.UNKNOWN, PlatformFlags.IS_64_BIT)
 
 
 class MockMemorySegment(NamedTuple):
@@ -448,3 +461,16 @@ def drgn_log_level(level: int):
         yield
     finally:
         logger.setLevel(old_level)
+
+
+@contextlib.contextmanager
+def with_default_prog(prog):
+    try:
+        old_default_prog = get_default_prog()
+    except NoDefaultProgramError:
+        old_default_prog = None
+    try:
+        set_default_prog(prog)
+        yield
+    finally:
+        set_default_prog(old_default_prog)
