@@ -11,10 +11,12 @@ import mmap
 import os
 from pathlib import Path
 import pickle
+import random
 import re
 import signal
 import socket
 import stat
+import string
 import struct
 import subprocess
 import sys
@@ -26,6 +28,30 @@ import unittest
 from _drgn_util.platform import _IO, _IOR, NORMALIZED_MACHINE_NAME, SYS
 import drgn
 from tests import TestCase
+from util import verrevcmp
+
+try:
+    import pyroute2
+
+    have_pyroute2 = True
+
+    # Before Pyroute2 commit 1eb08312de30 ("iproute/linux: try to improve flags
+    # when sending del messages") (in v0.6.10), Pyroute2 passes an invalid flag
+    # to deletion requests, resulting in ENOTSUP errors.
+    have_pyroute2_del = verrevcmp(getattr(pyroute2, "__version__", "0"), "0.6.10") >= 0
+
+    # Before Pyroute2 commit f0df9a49b41f ("Fix "failed to open netns" error in
+    # RISCV64.") (in v0.7.10), Pyroute2 uses the wrong syscall number on
+    # RISC-V. Monkey patch the correct number.
+    if verrevcmp(
+        getattr(pyroute2, "__version__", "0"), "0.7.10"
+    ) < 0 and os.uname().machine.startswith("riscv"):
+        import pyroute2.netns
+
+        pyroute2.netns.__NR_setns = 268
+except ImportError:
+    have_pyroute2 = False
+    have_pyroute2_del = False
 
 
 class LinuxKernelTestCase(TestCase):
@@ -152,6 +178,13 @@ skip_unless_have_memory_hotplug = unittest.skipUnless(
 )
 
 
+skip_unless_have_pyroute2 = unittest.skipUnless(have_pyroute2, "pyroute2 not found")
+
+skip_unless_have_pyroute2_del = unittest.skipUnless(
+    have_pyroute2_del, "pyroute2 >= 0.6.10 not found"
+)
+
+
 # PRNG used by the test kernel module.
 def prng32(seed):
     seed = seed.encode("ascii")
@@ -282,6 +315,22 @@ def meminfo_field_in_pages(name: str) -> int:
     )
 
 
+@contextlib.contextmanager
+def temp_netns():
+    while True:
+        try:
+            name = "".join(random.choice(string.ascii_letters) for _ in range(16))
+            ns = pyroute2.NetNS(name, flags=os.O_CREAT | os.O_EXCL)
+            break
+        except FileExistsError:
+            pass
+    try:
+        with ns:
+            yield name, ns
+    finally:
+        ns.remove()
+
+
 _c = ctypes.CDLL(None, use_errno=True)
 
 _mount = _c.mount
@@ -429,6 +478,23 @@ def mlock(addr, len):
     _check_ctypes_syscall(_mlock(addr, len))
 
 
+_prctl = _c.prctl
+_prctl.argtypes = [ctypes.c_int]
+_prctl.restype = ctypes.c_int
+
+
+def prctl_set_vma_anon_name(addr, size, name):
+    _check_ctypes_syscall(
+        _prctl(
+            ctypes.c_int(0x53564D41),  # PR_SET_VMA
+            ctypes.c_long(0),  # PR_SET_VMA_ANON_NAME,
+            ctypes.c_ulong(addr),
+            ctypes.c_ulong(size),
+            ctypes.c_char_p(None if name is None else os.fsencode(name)),
+        )
+    )
+
+
 CSIGNAL = 0x000000FF
 CLONE_VM = 0x00000100
 CLONE_FS = 0x00000200
@@ -737,3 +803,143 @@ def create_socket(*args, **kwds):
             raise unittest.SkipTest("kernel does not support TCP")
         else:
             raise
+
+
+IPC_PRIVATE = 0
+IPC_CREAT = 0o1000
+IPC_EXCL = 0o2000
+IPC_NOWAIT = 0o4000
+
+IPC_RMID = 0
+IPC_SET = 1
+IPC_STAT = 2
+IPC_INFO = 3
+
+_key_t = ctypes.c_int
+
+_ftok = _c.ftok
+_ftok.argtypes = [ctypes.c_char_p, ctypes.c_int]
+_ftok.restype = _key_t
+
+
+def ftok(path, proj_id: int) -> int:
+    return _check_ctypes_syscall(_ftok(os.fsencode(path), proj_id))
+
+
+_msgget = _c.msgget
+_msgget.argtypes = [_key_t, ctypes.c_int]
+_msgget.restype = ctypes.c_int
+
+
+def msgget(key: int, msgflg: int = IPC_CREAT | 0o666) -> int:
+    return _check_ctypes_syscall(_msgget(key, msgflg))
+
+
+_msgctl = _c.msgctl
+_msgctl.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.c_void_p]
+_msgctl.restype = ctypes.c_int
+
+
+def msgctl(msqid: int, op: int) -> None:
+    _check_ctypes_syscall(_msgctl(msqid, op, 0))
+
+
+_semget = _c.semget
+_semget.argtypes = [_key_t, ctypes.c_int, ctypes.c_int]
+_semget.restype = ctypes.c_int
+
+
+def semget(key: int, nsems: int, semflg: int = IPC_CREAT | 0o666) -> int:
+    return _check_ctypes_syscall(_semget(key, nsems, semflg))
+
+
+_semctl = _c.semctl
+_semctl.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.c_int]
+_semctl.restype = ctypes.c_int
+
+
+def semctl(semid: int, semnum: int, op: int) -> None:
+    _check_ctypes_syscall(_semctl(semid, semnum, op))
+
+
+SHM_LOCK = 11
+
+
+_shmget = _c.shmget
+_shmget.argtypes = [_key_t, ctypes.c_int, ctypes.c_int]
+_shmget.restype = ctypes.c_int
+
+
+def shmget(key: int, size: int, shmflg: int = IPC_CREAT | 0o666) -> int:
+    return _check_ctypes_syscall(_shmget(key, size, shmflg))
+
+
+_shmctl = _c.shmctl
+_shmctl.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.c_void_p]
+_shmctl.restype = ctypes.c_int
+
+
+def shmctl(shmid: int, op: int) -> None:
+    _check_ctypes_syscall(_shmctl(shmid, op, 0))
+
+
+_aio_context_t = ctypes.c_ulong
+
+
+def io_setup(nr_events):
+    ctx_id = _aio_context_t()
+    _check_ctypes_syscall(
+        _syscall(SYS["io_setup"], ctypes.c_uint(nr_events), ctypes.byref(ctx_id))
+    )
+    return ctx_id.value
+
+
+def io_destroy(ctx_id):
+    _check_ctypes_syscall(_syscall(SYS["io_destroy"], _aio_context_t(ctx_id)))
+
+
+_aio_key_and_aio_rw_flags = [
+    ("aio_key", ctypes.c_uint32),
+    ("aio_rw_flags", ctypes.c_int),
+]
+if sys.byteorder == "big":
+    _aio_key_and_aio_rw_flags.reverse()
+
+
+IOCB_CMD_PREAD = 0
+IOCB_CMD_PWRITE = 1
+IOCB_CMD_FSYNC = 2
+IOCB_CMD_FDSYNC = 3
+IOCB_CMD_POLL = 5
+IOCB_CMD_NOOP = 6
+IOCB_CMD_PREADV = 7
+IOCB_CMD_PWRITEV = 8
+
+
+class iocb(ctypes.Structure):
+    _fields_ = (
+        ("aio_data", ctypes.c_uint64),
+        *_aio_key_and_aio_rw_flags,
+        ("aio_lio_opcode", ctypes.c_uint16),
+        ("aio_reqprio", ctypes.c_int16),
+        ("aio_fildes", ctypes.c_uint32),
+        ("aio_buf", ctypes.c_uint64),
+        ("aio_nbytes", ctypes.c_uint64),
+        ("aio_offset", ctypes.c_int64),
+        ("aio_reserved2", ctypes.c_uint64),
+        ("aio_flags", ctypes.c_uint32),
+        ("aio_resfd", ctypes.c_uint32),
+    )
+
+
+def io_submit(ctx_id, iocbs):
+    nr = len(iocbs)
+    iocbp = (ctypes.POINTER(iocb) * nr)(*(ctypes.pointer(iocb) for iocb in iocbs))
+    return _check_ctypes_syscall(
+        _syscall(
+            SYS["io_submit"],
+            _aio_context_t(ctx_id),
+            ctypes.c_long(nr),
+            ctypes.byref(iocbp),
+        )
+    )
