@@ -1,34 +1,45 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates.
 # SPDX-License-Identifier: LGPL-2.1-or-later
 
+import contextlib
 import errno
 import os
 import resource
-import sys
 import unittest
 
 from _drgn_util.platform import NORMALIZED_MACHINE_NAME
+from drgn import cast
 from drgn.helpers.linux.bpf import (
     bpf_btf_for_each,
     bpf_link_for_each,
+    bpf_map_by_id,
     bpf_map_for_each,
+    bpf_prog_by_id,
     bpf_prog_for_each,
+    bpf_prog_used_maps,
     cgroup_bpf_prog_for_each,
     cgroup_bpf_prog_for_each_effective,
 )
 from drgn.helpers.linux.cgroup import cgroup_get_from_path
+from drgn.helpers.linux.fs import fget
+from drgn.helpers.linux.pid import find_task
 from tests.linux_kernel import LinuxKernelTestCase
 from tests.linux_kernel.bpf import (
     BPF_CGROUP_INET_INGRESS,
+    BPF_EXIT_INSN,
     BPF_F_ALLOW_MULTI,
+    BPF_LD_MAP_FD,
     BPF_MAP_TYPE_HASH,
+    BPF_MOV64_IMM,
     BPF_PROG_TYPE_CGROUP_SKB,
     BPF_PROG_TYPE_SOCKET_FILTER,
+    BPF_REG_0,
     _SYS_bpf,
     bpf_btf_ids,
     bpf_link_create,
     bpf_link_ids,
     bpf_map_create,
+    bpf_map_get_info_by_fd,
     bpf_map_ids,
     bpf_prog_attach,
     bpf_prog_get_info_by_fd,
@@ -38,14 +49,11 @@ from tests.linux_kernel.bpf import (
 from tests.linux_kernel.helpers.test_cgroup import tmp_cgroups
 
 
-class TestBpf(LinuxKernelTestCase):
-    # BPF instructions for:
-    # r0 = 0
-    # exit
-    if sys.byteorder == "little":
-        INSNS = (0xB7, 0x95)
-    else:
-        INSNS = (0xB700000000000000, 0x9500000000000000)
+class BpfTestCase(LinuxKernelTestCase):
+    INSNS = (
+        BPF_MOV64_IMM(BPF_REG_0, 0),
+        BPF_EXIT_INSN(),
+    )
 
     @classmethod
     def setUpClass(cls):
@@ -78,6 +86,8 @@ class TestBpf(LinuxKernelTestCase):
                 "kernel does not support bpf syscall (CONFIG_BPF_SYSCALL)"
             )
 
+
+class TestBpf(BpfTestCase):
     def test_bpf_btf_for_each(self):
         # BTF was added in Linux kernel commit 69b693f0aefa ("bpf: btf:
         # Introduce BPF Type Format (BTF)") (in v4.18) and had IDs from the
@@ -209,6 +219,30 @@ class TestBpf(LinuxKernelTestCase):
             for fd in fds:
                 os.close(fd)
 
+    def test_bpf_prog_used_maps(self):
+        with contextlib.ExitStack() as exit_stack:
+            map_fd = bpf_map_create(BPF_MAP_TYPE_HASH, 8, 8, 8)
+            exit_stack.callback(os.close, map_fd)
+
+            prog_fd = bpf_prog_load(
+                BPF_PROG_TYPE_SOCKET_FILTER,
+                BPF_LD_MAP_FD(BPF_REG_0, map_fd) + self.INSNS,
+                b"GPL",
+            )
+            exit_stack.callback(os.close, prog_fd)
+
+            bpf_prog = cast(
+                "struct bpf_prog *",
+                fget(find_task(self.prog, os.getpid()), prog_fd).private_data,
+            )
+
+            bpf_map = cast(
+                "struct bpf_map *",
+                fget(find_task(self.prog, os.getpid()), map_fd).private_data,
+            )
+
+            self.assertEqual(list(bpf_prog_used_maps(bpf_prog)), [bpf_map])
+
     def test_cgroup_bpf_prog_for_each(self):
         with tmp_cgroups() as (parent_cgroup, child_cgroup):
             fds = []
@@ -312,3 +346,47 @@ class TestBpf(LinuxKernelTestCase):
             finally:
                 for fd in fds:
                     os.close(fd)
+
+    def test_bpf_prog_by_id(self):
+        fd = bpf_prog_load(BPF_PROG_TYPE_SOCKET_FILTER, self.INSNS, b"GPL")
+        self.addCleanup(os.close, fd)
+        try:
+            prog_id = bpf_prog_get_info_by_fd(fd).id
+        except OSError as e:
+            # bpf_prog_by_id() and BPF program IDs aren't supported before
+            # Linux v4.13, which added IDs for BPF programs in commit
+            # dc4bb0e23561 ("bpf: Introduce bpf_prog ID") and an API to get
+            # them in commit 34ad5580f8f9 ("bpf: Add BPF_(PROG|MAP)_GET_NEXT_ID
+            # command").
+            if e.errno != errno.EINVAL:
+                raise
+            self.skipTest("kernel does not support BPF_PROG_GET_NEXT_ID")
+
+        prog = bpf_prog_by_id(self.prog, prog_id)
+        self.assertNotEqual(prog.value_(), 0)
+        self.assertEqual(prog.aux.id.value_(), prog_id)
+
+        non_existent_prog = bpf_prog_by_id(self.prog, 0x7FFFFFFF)
+        self.assertEqual(non_existent_prog.value_(), 0)
+
+    def test_bpf_map_by_id(self):
+        fd = bpf_map_create(BPF_MAP_TYPE_HASH, 8, 8, 8)
+        self.addCleanup(os.close, fd)
+        try:
+            map_id = bpf_map_get_info_by_fd(fd).id
+        except OSError as e:
+            # bpf_map_by_id() and BPF map IDs aren't supported before
+            # Linux v4.13, which added IDs for BPF maps in commit
+            # f3f1c054c288 ("bpf: Introduce bpf_map ID") and an API to get
+            # them in commit 34ad5580f8f9 ("bpf: Add BPF_(PROG|MAP)_GET_NEXT_ID
+            # command").
+            if e.errno != errno.EINVAL:
+                raise
+            self.skipTest("kernel does not support BPF_MAP_GET_NEXT_ID")
+
+        bpf_map = bpf_map_by_id(self.prog, map_id)
+        self.assertNotEqual(bpf_map.value_(), 0)
+        self.assertEqual(bpf_map.id.value_(), map_id)
+
+        non_existent_map = bpf_map_by_id(self.prog, 0x7FFFFFFF)
+        self.assertEqual(non_existent_map.value_(), 0)
