@@ -2,7 +2,8 @@
 # SPDX-License-Identifier: LGPL-2.1-or-later
 
 from drgn import Object
-from drgn.helpers.linux.kthread import kthread_data
+from drgn.helpers.linux.kthread import kthread_data, task_is_kthread
+from drgn.helpers.linux.pid import find_task
 from tests.linux_kernel import LinuxKernelTestCase, skip_unless_have_test_kmod
 
 
@@ -15,3 +16,7 @@ class TestKthread(LinuxKernelTestCase):
             kthread_data(self.prog["drgn_test_kthread"]),
             Object(self.prog, "void *", 0xB0BA000),
         )
+
+    def test_task_is_kthread(self):
+        self.assertTrue(task_is_kthread(self.prog["init_task"].address_of_()))
+        self.assertFalse(task_is_kthread(find_task(self.prog, 1)))
