@@ -3,6 +3,7 @@
 
 import contextlib
 import ctypes
+import errno
 import mmap
 import os
 from pathlib import Path
@@ -16,7 +17,6 @@ from _drgn_util.platform import NORMALIZED_MACHINE_NAME
 from drgn import NULL, FaultError, ObjectNotFoundError
 from drgn.helpers.linux.device import dev_name
 from drgn.helpers.linux.mm import (
-    _MEMORY_BLOCK_STATE,
     PFN_PHYS,
     PHYS_PFN,
     PageCompound,
@@ -31,6 +31,7 @@ from drgn.helpers.linux.mm import (
     compound_nr,
     compound_order,
     decode_memory_block_state,
+    decode_memory_block_state_value,
     decode_page_flags,
     environ,
     find_vmap_area,
@@ -52,6 +53,7 @@ from drgn.helpers.linux.mm import (
     phys_to_page,
     phys_to_virt,
     task_rss,
+    task_vsize,
     totalram_pages,
     virt_to_page,
     virt_to_pfn,
@@ -59,6 +61,7 @@ from drgn.helpers.linux.mm import (
     vm_commit_limit,
     vm_memory_committed,
     vma_find,
+    vma_name,
     vmalloc_to_page,
     vmalloc_to_pfn,
 )
@@ -69,6 +72,7 @@ from tests.linux_kernel import (
     iter_maps,
     meminfo_field_in_pages,
     mlock,
+    prctl_set_vma_anon_name,
     prng32,
     skip_if_highmem,
     skip_if_highpte,
@@ -78,16 +82,11 @@ from tests.linux_kernel import (
 )
 
 
-class TestMm(LinuxKernelTestCase):
-    def test_page_constants(self):
-        self.assertEqual(self.prog["PAGE_SIZE"], mmap.PAGESIZE)
-        self.assertEqual(1 << self.prog["PAGE_SHIFT"], mmap.PAGESIZE)
-        self.assertEqual(~self.prog["PAGE_MASK"] + 1, mmap.PAGESIZE)
-
+class MmTestCase(LinuxKernelTestCase):
     # Returns an mmap.mmap object for a file mapping in /dev/shm, its mapped
     # address, and the pfns backing it.
     @contextlib.contextmanager
-    def _pages(self):
+    def map_pages(self):
         if not os.path.exists("/proc/self/pagemap"):
             self.skipTest("kernel does not support pagemap")
 
@@ -109,12 +108,19 @@ class TestMm(LinuxKernelTestCase):
                     ]
                 yield map, address, pfns
 
+
+class TestMm(MmTestCase):
+    def test_page_constants(self):
+        self.assertEqual(self.prog["PAGE_SIZE"], mmap.PAGESIZE)
+        self.assertEqual(1 << self.prog["PAGE_SHIFT"], mmap.PAGESIZE)
+        self.assertEqual(~self.prog["PAGE_MASK"] + 1, mmap.PAGESIZE)
+
     def test_page_index(self):
-        with self._pages() as (_, _, pfns):
+        with self.map_pages() as (_, _, pfns):
             self.assertEqual(page_index(pfn_to_page(self.prog, pfns[3])), 3)
 
     def test_page_flag_getters(self):
-        with self._pages() as (map, _, pfns):
+        with self.map_pages() as (map, _, pfns):
             page = pfn_to_page(self.prog, pfns[0])
             # The page flag getters are generated, so just pick a positive case
             # and a negative case to cover all of them.
@@ -172,10 +178,14 @@ class TestMm(LinuxKernelTestCase):
 
     @skip_unless_have_full_mm_support
     def test_decode_page_flags(self):
-        with self._pages() as (map, _, pfns):
+        with self.map_pages() as (map, _, pfns):
             page = pfn_to_page(self.prog, pfns[0])
             self.assertIn("PG_swapbacked", decode_page_flags(page))
 
+    # In principle, this shouldn't need full MM support, but /proc/kcore is
+    # sometimes missing some parts of vmemmap, requiring virtual address
+    # translation.
+    @skip_unless_have_full_mm_support
     @skip_unless_have_test_kmod
     def test_for_each_valid_page_range(self):
         expected_pfn = self.prog["drgn_test_pfn"].value_()
@@ -300,7 +310,7 @@ class TestMm(LinuxKernelTestCase):
     @skip_if_highpte
     def test_follow_pfn(self):
         task = find_task(self.prog, os.getpid())
-        with self._pages() as (map, address, pfns):
+        with self.map_pages() as (map, address, pfns):
             self.assertEqual(follow_pfn(task.mm, address), pfns[0])
 
     @skip_unless_have_full_mm_support
@@ -460,6 +470,48 @@ class TestMm(LinuxKernelTestCase):
                 vma_find(mm, prev_end), NULL(self.prog, "struct vm_area_struct *")
             )
 
+    def test_vma_name(self):
+        with mmap.mmap(-1, mmap.PAGESIZE, mmap.MAP_PRIVATE) as private_map, mmap.mmap(
+            -1, mmap.PAGESIZE, mmap.MAP_SHARED
+        ) as shared_map:
+            # Test VMA names if the kernel supports it.
+            try:
+                prctl_set_vma_anon_name(
+                    ctypes.addressof(ctypes.c_char.from_buffer(private_map)),
+                    mmap.PAGESIZE,
+                    "testprivate",
+                )
+            except OSError as e:
+                # PR_SET_VMA_ANON_NAME is only supported since Linux 5.17, and
+                # only if CONFIG_ANON_VMA_NAME=y. Otherwise, it returns EINVAL.
+                if e.errno != errno.EINVAL:
+                    raise
+            try:
+                prctl_set_vma_anon_name(
+                    ctypes.addressof(ctypes.c_char.from_buffer(shared_map)),
+                    mmap.PAGESIZE,
+                    "testshared",
+                )
+            except OSError as e:
+                # Unsupported VMAs return EBADF, and anonymous shared memory is
+                # only supported since Linux 6.2.
+                if e.errno != errno.EINVAL and e.errno != errno.EBADF:
+                    raise
+
+            mm = find_task(self.prog, os.getpid()).mm.read_()
+            tested_file_path = False
+            for map in iter_maps():
+                if map.path.startswith("["):
+                    vma = vma_find(mm, map.start)
+                    if vma:
+                        with self.subTest(vma=map.path):
+                            self.assertEqual(vma_name(vma), os.fsencode(map.path))
+                elif not tested_file_path and map.path.startswith("/"):
+                    vma = vma_find(mm, map.start)
+                    with self.subTest("file"):
+                        self.assertEqual(vma_name(vma), os.fsencode(map.path))
+                    tested_file_path = True
+
     def test_for_each_vma(self):
         with fork_and_stop() as pid:
             self.assertEqual(
@@ -555,6 +607,15 @@ class TestMm(LinuxKernelTestCase):
             # margin.
             self.assertAlmostEqual(rss_info.total, stats["VmRSS"], delta=delta * 3)
 
+    def test_task_vsize(self):
+        with fork_and_stop() as pid:
+            task = find_task(self.prog, pid)
+            vsize = task_vsize(task)
+            text = Path(f"/proc/{pid}/status").read_text()
+            value = re.findall(r"^VmSize:\s*([0-9]+)", text, flags=re.MULTILINE)
+            if value:
+                self.assertEqual(vsize, int(value[0]) * 1024)
+
     @skip_unless_have_memory_hotplug
     def test_for_each_memory_block(self):
         self.assertCountEqual(
@@ -565,16 +626,6 @@ class TestMm(LinuxKernelTestCase):
             os.listdir(b"/sys/bus/memory/devices"),
         )
 
-    @skip_unless_have_test_kmod
-    def test_memory_block_states(self):
-        for value, name in _MEMORY_BLOCK_STATE.items():
-            with self.subTest(state=name):
-                try:
-                    expected = self.prog["drgn_test_" + name].value_()
-                except ObjectNotFoundError:
-                    self.skipTest(f"{name} is not defined")
-                self.assertEqual(value, expected)
-
     @skip_unless_have_memory_hotplug
     def test_decode_memory_block_state(self):
         mem = next(iter(for_each_memory_block(self.prog)))
@@ -592,6 +643,25 @@ class TestMm(LinuxKernelTestCase):
             .strip(),
         )
 
+    @skip_unless_have_test_kmod
+    def test_decode_memory_block_state_value(self):
+        for name in (
+            "MEM_ONLINE",
+            "MEM_GOING_OFFLINE",
+            "MEM_OFFLINE",
+            "MEM_GOING_ONLINE",
+            "MEM_CANCEL_ONLINE",
+            "MEM_CANCEL_OFFLINE",
+            "MEM_PREPARE_ONLINE",
+            "MEM_FINISH_OFFLINE",
+        ):
+            with self.subTest(state=name):
+                try:
+                    state = self.prog["drgn_test_" + name]
+                except ObjectNotFoundError:
+                    self.skipTest(f"{name} is not defined")
+                self.assertEqual(decode_memory_block_state_value(state), name)
+
     @skip_unless_have_memory_hotplug
     def test_memory_block_size_bytes(self):
         self.assertEqual(
