@@ -1,6 +1,7 @@
 # Copyright (c) Meta Platforms, Inc. and affiliates.
 # SPDX-License-Identifier: LGPL-2.1-or-later
 
+import ipaddress
 import os
 import socket
 import tempfile
@@ -13,11 +14,15 @@ from drgn.helpers.linux.net import (
     SOCK_INODE,
     SOCKET_I,
     for_each_net,
+    for_each_netdev,
     get_net_ns_by_fd,
     is_pp_page,
     netdev_for_each_tx_queue,
     netdev_get_by_index,
     netdev_get_by_name,
+    netdev_ipv4_addrs,
+    netdev_ipv6_addrs,
+    netdev_name,
     netdev_priv,
     sk_fullsock,
     skb_shinfo,
@@ -26,6 +31,7 @@ from drgn.helpers.linux.pid import find_task
 from tests.linux_kernel import (
     LinuxKernelTestCase,
     create_socket,
+    skip_unless_have_pyroute2,
     skip_unless_have_test_kmod,
 )
 from util import KernelVersion
@@ -46,6 +52,20 @@ class TestNet(LinuxKernelTestCase):
             sk = cast("struct socket *", file.private_data).sk.read_()
             self.assertTrue(sk_fullsock(sk))
 
+    def test_for_each_netdev(self):
+        self.assertCountEqual(
+            [netdev_name(dev) for dev in for_each_netdev(self.net)],
+            [name.encode() for _, name in socket.if_nameindex()],
+        )
+
+    @skip_unless_have_test_kmod
+    def test_for_each_netdev_init_net(self):
+        self.assertIn(self.prog["drgn_test_netdev"], for_each_netdev(self.prog))
+
+    @skip_unless_have_test_kmod
+    def test_netdev_name(self):
+        self.assertEqual(netdev_name(self.prog["drgn_test_netdev"]), b"lo")
+
     def test_netdev_get_by_index(self):
         for index, name in socket.if_nameindex():
             netdev = netdev_get_by_index(self.net, index)
@@ -132,3 +152,33 @@ class TestNet(LinuxKernelTestCase):
             self.skipTest("kernel does not support identifying page_pool pages")
         self.assertTrue(is_pp_page(self.prog["drgn_test_page_pool_page"]))
         self.assertFalse(is_pp_page(self.prog["drgn_test_page"]))
+
+    @skip_unless_have_pyroute2
+    def test_netdev_ipv4_addrs(self):
+        import pyroute2
+
+        with pyroute2.IPRoute() as ipr:
+            idx = ipr.link_lookup(ifname="lo")[0]
+            addrs = ipr.get_addr(index=idx, family=socket.AF_INET)
+            expected = [
+                ipaddress.IPv4Address(addr.get_attr("IFA_ADDRESS")) for addr in addrs
+            ]
+        self.assertCountEqual(
+            netdev_ipv4_addrs(netdev_get_by_name(self.prog, "lo")),
+            expected,
+        )
+
+    @skip_unless_have_pyroute2
+    def test_netdev_ipv6_addrs(self):
+        import pyroute2
+
+        with pyroute2.IPRoute() as ipr:
+            idx = ipr.link_lookup(ifname="lo")[0]
+            addrs = ipr.get_addr(index=idx, family=socket.AF_INET6)
+            expected = [
+                ipaddress.IPv6Address(addr.get_attr("IFA_ADDRESS")) for addr in addrs
+            ]
+        self.assertCountEqual(
+            netdev_ipv6_addrs(netdev_get_by_name(self.prog, "lo")),
+            expected,
+        )
