@@ -15,13 +15,14 @@ from drgn.commands import (
     CommandNamespace,
     CommandNotFoundError,
     DrgnCodeBuilder,
-    _parse_shell_command,
-    _ParsedShellCommand,
+    ParsedCommand,
     _repr_black,
+    _repr_raw_bytes,
     _sanitize_rst,
     argument,
     command,
-    custom_command,
+    parse_shell_command,
+    raw_command,
 )
 from tests import TestCase
 
@@ -29,10 +30,8 @@ from tests import TestCase
 class TestParseShellCommand(TestCase):
     def assert_parses_to(self, source, *, args, redirections=[], pipeline=None):
         self.assertEqual(
-            _parse_shell_command(source),
-            _ParsedShellCommand(
-                args=args, redirections=redirections, pipeline=pipeline
-            ),
+            parse_shell_command(source),
+            ParsedCommand(args=args, redirections=redirections, pipeline=pipeline),
         )
 
     def test_one_arg(self):
@@ -245,7 +244,7 @@ class TestParseShellCommand(TestCase):
 
     def test_pipeline_empty(self):
         self.assertRaisesRegex(
-            SyntaxError, "unexpected end of input", _parse_shell_command, "foo | "
+            SyntaxError, "unexpected end of input", parse_shell_command, "foo | "
         )
 
     def test_pipe_after_redirect(self):
@@ -255,7 +254,7 @@ class TestParseShellCommand(TestCase):
             "foo >> |",
         ):
             self.assertRaisesRegex(
-                SyntaxError, r"unexpected '\|'", _parse_shell_command, source
+                SyntaxError, r"unexpected '\|'", parse_shell_command, source
             )
 
     def test_nothing_before_pipe(self):
@@ -263,21 +262,21 @@ class TestParseShellCommand(TestCase):
 
     def test_redirect_after_redirect(self):
         self.assertRaisesRegex(
-            SyntaxError, "unexpected '<'", _parse_shell_command, "foo < <"
+            SyntaxError, "unexpected '<'", parse_shell_command, "foo < <"
         )
         self.assertRaisesRegex(
-            SyntaxError, "unexpected '>'", _parse_shell_command, "foo < >"
+            SyntaxError, "unexpected '>'", parse_shell_command, "foo < >"
         )
         self.assertRaisesRegex(
-            SyntaxError, "unexpected '>>'", _parse_shell_command, "foo < >>"
+            SyntaxError, "unexpected '>>'", parse_shell_command, "foo < >>"
         )
 
     def test_nothing_after_redirect(self):
         self.assertRaisesRegex(
-            SyntaxError, "unexpected end of input", _parse_shell_command, ">"
+            SyntaxError, "unexpected end of input", parse_shell_command, ">"
         )
         self.assertRaisesRegex(
-            SyntaxError, "unexpected end of input", _parse_shell_command, "foo >"
+            SyntaxError, "unexpected end of input", parse_shell_command, "foo >"
         )
 
     def test_comment(self):
@@ -318,7 +317,7 @@ def _cmd_tac(prog, name, args, **kwargs):
     sys.stdout.writelines(lines)
 
 
-@custom_command(
+@raw_command(
     description="call ast.literal_eval()",
     usage="literal expr",
     long_description="",
@@ -482,7 +481,7 @@ class TestRunCommand(TestCase):
 
         self.assertRaises(ZeroDivisionError, self.run_command, "asdf", onerror=onerror)
 
-    def test_custom(self):
+    def test_raw(self):
         self.assertEqual(self.run_command("literal (1, 2)"), (1, 2))
 
     def test_argument_error(self):
@@ -528,17 +527,43 @@ class TestReprBlack(TestCase):
         self.assertEqual(_repr_black('l"oL'), "'l\"oL'")
 
 
+class TestReprRawBytes(TestCase):
+    def test_simple(self):
+        self.assertEqual(_repr_raw_bytes(b"foo"), 'rb"foo"')
+
+    def test_single_quote(self):
+        self.assertEqual(_repr_raw_bytes(b"ne'er"), 'rb"ne\'er"')
+
+    def test_double_quote(self):
+        self.assertEqual(_repr_raw_bytes(b'l"oL'), "rb'l\"oL'")
+
+    def test_single_and_double_quote(self):
+        self.assertEqual(_repr_raw_bytes(b"ne'er l\"oL"), 'b"ne\'er l\\"oL"')
+
+    def test_ends_with_backslash(self):
+        self.assertEqual(_repr_raw_bytes(b"foo\\"), r'b"foo\\"')
+
+    def test_single_quote_and_ends_with_backslash(self):
+        self.assertEqual(_repr_raw_bytes(b"ne'er\\"), 'b"ne\'er\\\\"')
+
+    def test_double_quote_and_ends_with_backslash(self):
+        self.assertEqual(_repr_raw_bytes(b'l"oL\\'), "b'l\"oL\\\\'")
+
+    def test_single_and_double_quote_and_ends_with_backslash(self):
+        self.assertEqual(_repr_raw_bytes(b"ne'er l\"oL\\"), 'b"ne\'er l\\"oL\\\\"')
+
+
 class TestDrgnCodeBuilder(TestCase):
     def test_empty(self):
-        self.assertEqual(DrgnCodeBuilder().get(), "")
+        self.assertEqual(DrgnCodeBuilder(None).get(), "")
 
     def test_no_imports(self):
-        code = DrgnCodeBuilder()
+        code = DrgnCodeBuilder(None)
         code.append("pass\n")
         self.assertEqual(code.get(), "pass\n")
 
     def test_sort_imports(self):
-        code = DrgnCodeBuilder()
+        code = DrgnCodeBuilder(None)
         code.add_import("sys")
         code.add_import("os")
         self.assertEqual(
@@ -550,7 +575,7 @@ import sys
         )
 
     def test_deduplicate_imports(self):
-        code = DrgnCodeBuilder()
+        code = DrgnCodeBuilder(None)
         code.add_import("os")
         code.add_import("sys")
         code.add_import("os")
@@ -563,7 +588,7 @@ import sys
         )
 
     def test_sort_from_imports(self):
-        code = DrgnCodeBuilder()
+        code = DrgnCodeBuilder(None)
         code.add_import("sys")
         code.add_import("os")
         code.add_from_import("sys", "stdout")
@@ -577,7 +602,7 @@ from sys import stdout
         )
 
     def test_first_party_imports(self):
-        code = DrgnCodeBuilder()
+        code = DrgnCodeBuilder(None)
         code.add_import("os")
         code.add_import("sys")
         code.add_import("drgn")
@@ -592,7 +617,7 @@ import drgn
         )
 
     def test_merge_from_imports(self):
-        code = DrgnCodeBuilder()
+        code = DrgnCodeBuilder(None)
         code.add_from_import("drgn", "Object", "Program")
         code.add_from_import("drgn", "Object")
         code.add_from_import("drgn", "Type")
@@ -604,7 +629,7 @@ from drgn import Object, Program, Type
         )
 
     def test_long_from_imports(self):
-        code = DrgnCodeBuilder()
+        code = DrgnCodeBuilder(None)
         code.add_from_import(
             "os",
             "abort",
@@ -635,7 +660,7 @@ from os import (
         )
 
     def test_imports_and_code(self):
-        code = DrgnCodeBuilder()
+        code = DrgnCodeBuilder(None)
         code.append("pass\n")
         code.add_import("os")
         self.assertEqual(
