@@ -8,6 +8,7 @@ import struct
 from drgn import (
     AbsenceReason,
     FaultError,
+    NoDefaultProgramError,
     Object,
     ObjectAbsentError,
     OutOfBoundsError,
@@ -19,9 +20,11 @@ from drgn import (
 )
 from tests import (
     MockMemorySegment,
+    MockObject,
     MockProgramTestCase,
     assertReprPrettyEqualsStr,
     mock_program,
+    with_default_prog,
 )
 
 
@@ -2337,6 +2340,12 @@ class TestGenericOperators(MockProgramTestCase):
                 )
         self.assertRaises(TypeError, obj.format_, integer_base="hex")
 
+    def test_sizeof_default_prog(self):
+        self.objects.append(MockObject("foo", self.prog.int_type("int", 4, True), 1))
+        self.assertRaises(NoDefaultProgramError, sizeof, "foo")
+        with with_default_prog(self.prog):
+            self.assertEqual(sizeof("foo"), 4)
+
 
 class TestSpecialMethods(MockProgramTestCase):
     def test_dir(self):
@@ -2369,7 +2378,6 @@ class TestSpecialMethods(MockProgramTestCase):
         obj = Object(self.prog, "int [4]", value=[0, 1, 2, 3])
         for i, element in enumerate(obj):
             self.assertIdentical(element, Object(self.prog, "int", value=i))
-        self.assertEqual(operator.length_hint(iter(obj)), 4)
         self.assertRaisesRegex(
             TypeError, "'int' is not iterable", iter, Object(self.prog, "int", value=0)
         )
@@ -2380,6 +2388,37 @@ class TestSpecialMethods(MockProgramTestCase):
             Object(self.prog, "int []", address=0),
         )
 
+    def test_iter_length_hint(self):
+        it = iter(Object(self.prog, "int [3]", value=[0, 1, 2]))
+        for i in range(3, 0, -1):
+            self.assertEqual(operator.length_hint(it), i)
+            next(it)
+        self.assertEqual(operator.length_hint(it), 0)
+
+    def test_reversed(self):
+        obj = Object(self.prog, "int [4]", value=[0, 1, 2, 3])
+        for i, element in zip(range(3, -1, -1), reversed(obj)):
+            self.assertIdentical(element, Object(self.prog, "int", value=i))
+        self.assertRaisesRegex(
+            TypeError,
+            "'int' is not iterable",
+            reversed,
+            Object(self.prog, "int", value=0),
+        )
+        self.assertRaisesRegex(
+            TypeError,
+            r"'int \[\]' is not iterable",
+            reversed,
+            Object(self.prog, "int []", address=0),
+        )
+
+    def test_reversed_length_hint(self):
+        it = reversed(Object(self.prog, "int [3]", value=[0, 1, 2]))
+        for i in range(3, 0, -1):
+            self.assertEqual(operator.length_hint(it), i)
+            next(it)
+        self.assertEqual(operator.length_hint(it), 0)
+
     def test__repr_pretty_(self):
         obj = Object(self.prog, "int", value=0)
         assertReprPrettyEqualsStr(obj)
