@@ -28,6 +28,7 @@ from typing import (
 )
 
 from util import KernelVersion, nproc
+from vmtest.chroot import chroot_sh_cmd
 from vmtest.config import (
     ARCHITECTURES,
     HOST_ARCHITECTURE,
@@ -38,7 +39,7 @@ from vmtest.config import (
     Kernel,
 )
 from vmtest.download import Downloader
-from vmtest.rootfsbuild import build_drgn_in_rootfs
+from vmtest.rootfsbuild import build_drgn_for_arch
 from vmtest.vm import LostVMError, TestKmodMode, run_in_vm
 
 logger = logging.getLogger(__name__)
@@ -55,20 +56,6 @@ class _TestFunction(Protocol):
     def __call__(self, *, outfile: Optional[TextIO] = None) -> bool: ...
 
 
-def _kernel_version_is_supported(version: str, arch: Architecture) -> bool:
-    # /proc/kcore is broken on AArch64 and Arm on older versions.
-    if arch.name in ("aarch64", "arm") and KernelVersion(version) < KernelVersion(
-        "4.19"
-    ):
-        return False
-    # Before 4.11, we need an implementation of the
-    # linux_kernel_live_direct_mapping_fallback architecture callback in
-    # libdrgn, which we only have for x86_64.
-    if KernelVersion(version) < KernelVersion("4.11") and arch.name != "x86_64":
-        return False
-    return True
-
-
 def _kdump_works(kernel: Kernel) -> bool:
     if kernel.arch.name == "aarch64":
         # kexec fails with "kexec: setup_2nd_dtb failed." on older versions.
@@ -85,7 +72,7 @@ def _kdump_works(kernel: Kernel) -> bool:
     elif kernel.arch.name == "s390x":
         # Before 5.15, sysrq-c hangs.
         return KernelVersion(kernel.release) >= KernelVersion("5.15")
-    elif kernel.arch.name == "x86_64":
+    elif kernel.arch.name in {"x86_64", "riscv64"}:
         return True
     else:
         assert False, kernel.arch.name
@@ -113,6 +100,8 @@ class _TestRunner:
         jobs: Optional[int] = None,
         use_host_rootfs: bool = True,
         skip_build: bool = False,
+        pytest_kernel_args: Optional[str] = None,
+        skip_kdump: bool = False,
     ) -> None:
         self._directory = directory
         if jobs is None:
@@ -148,6 +137,9 @@ class _TestRunner:
         except (AttributeError, OSError):
             self._color = False
 
+        self._pytest_kernel_args = pytest_kernel_args
+        self._skip_kdump = skip_kdump
+
     def add_kernel(self, arch: Architecture, pattern: str) -> None:
         self._compilers_to_resolve[arch] = None
         self._kernels_to_resolve[(arch, pattern)] = None
@@ -312,15 +304,14 @@ class _TestRunner:
                 outfile = exit_stack.enter_context(
                     (self._directory / "log" / f"{arch.name}-build.log").open("w")
                 )
-            rootfs = self._rootfs(arch)
-            if rootfs == Path("/"):
+            if self._use_host_rootfs and arch is HOST_ARCHITECTURE:
                 subprocess.check_call(
                     [sys.executable, "setup.py", "build_ext", "-i"],
                     stdout=outfile,
                     stderr=outfile,
                 )
             else:
-                build_drgn_in_rootfs(rootfs, outfile=outfile)
+                build_drgn_for_arch(arch, self._directory, outfile=outfile)
         return functools.partial(self._drgn_build_done, arch)
 
     def _drgn_build_done(self, arch: Architecture) -> bool:
@@ -436,18 +427,17 @@ class _TestRunner:
             args = [
                 "unshare",
                 "--map-root-user",
-                "--map-users=auto",
-                "--map-groups=auto",
+                "--map-auto",
                 "--fork",
                 "--pid",
                 "--mount-proc=" + str(rootfs / "proc"),
                 "sh",
                 "-c",
-                """\
+                f"""\
 set -e
 
 mount --bind . "$1/mnt"
-chroot "$1" sh -c 'cd /mnt && pytest -v --ignore=tests/linux_kernel'
+{chroot_sh_cmd('"$1"')} 'cd /mnt && pytest -v --ignore=tests/linux_kernel'
 """,
                 "sh",
                 str(rootfs),
@@ -461,13 +451,7 @@ chroot "$1" sh -c 'cd /mnt && pytest -v --ignore=tests/linux_kernel'
         else:
             python_executable = "/usr/bin/python3"
 
-        if kernel.arch is HOST_ARCHITECTURE:
-            mark_expression = ""
-        else:
-            # Skip excessively slow tests when emulating.
-            mark_expression = "-m 'not slow'"
-
-        if _kdump_works(kernel):
+        if _kdump_works(kernel) and not self._skip_kdump:
             kdump_command = """\
     "$PYTHON" -Bm vmtest.enter_kdump
     # We should crash and not reach this.
@@ -476,6 +460,15 @@ chroot "$1" sh -c 'cd /mnt && pytest -v --ignore=tests/linux_kernel'
         else:
             kdump_command = ""
 
+        if self._pytest_kernel_args is not None:
+            pytest_args = self._pytest_kernel_args
+        elif kernel.arch is HOST_ARCHITECTURE:
+            pytest_args = "tests/linux_kernel --ignore tests/linux_kernel/vmcore"
+        else:
+            pytest_args = (
+                "tests/linux_kernel --ignore tests/linux_kernel/vmcore -m 'not slow'"
+            )
+
         test_command = rf"""
 set -e
 
@@ -485,11 +478,17 @@ if [ -e /proc/vmcore ]; then
     "$PYTHON" -Bm pytest -v tests/linux_kernel/vmcore
 else
     insmod "$DRGN_TEST_KMOD"
-    "$PYTHON" -Bm pytest -v tests/linux_kernel --ignore=tests/linux_kernel/vmcore {mark_expression}
+    "$PYTHON" -Bm pytest -v {pytest_args}
 {kdump_command}
 fi
 """
 
+        # Silence as much boot text as possible if we're specifying tests to run
+        # in the foreground
+        extra_kernel_cmdline = []
+        if self._pytest_kernel_args and self._foreground:
+            extra_kernel_cmdline.append("quiet")
+
         try:
             status = run_in_vm(
                 test_command,
@@ -498,6 +497,7 @@ fi
                 self._directory,
                 test_kmod=TestKmodMode.BUILD,
                 outfile=outfile,
+                extra_kernel_cmdline=extra_kernel_cmdline,
             )
             return status == 0
         except (
@@ -579,6 +579,15 @@ if __name__ == "__main__":
         action="store_true",
         help="don't rebuild drgn even if it's out of date",
     )
+    parser.add_argument(
+        "--pytest-args",
+        help="a string of args passed to pytest for kernel tests",
+    )
+    parser.add_argument(
+        "--skip-kdump",
+        action="store_true",
+        help="skip kdump and vmcore tests (useful with --pytest-args)",
+    )
     args = parser.parse_args()
 
     if not hasattr(args, "kernels") and not args.local:
@@ -632,6 +641,8 @@ if __name__ == "__main__":
         jobs=args.jobs,
         use_host_rootfs=args.use_host_rootfs == "auto",
         skip_build=args.skip_build,
+        pytest_kernel_args=args.pytest_args,
+        skip_kdump=args.skip_kdump,
     )
 
     if hasattr(args, "kernels"):
@@ -639,14 +650,14 @@ if __name__ == "__main__":
             if pattern == "all":
                 for version in SUPPORTED_KERNEL_VERSIONS:
                     for arch in architectures:
-                        if _kernel_version_is_supported(version, arch):
+                        if arch.kernel_version_supported(version):
                             for flavor in KERNEL_FLAVORS.values():
                                 runner.add_kernel(arch, version + ".*" + flavor.name)
             elif pattern in KERNEL_FLAVORS:
                 flavor = KERNEL_FLAVORS[pattern]
                 for version in SUPPORTED_KERNEL_VERSIONS:
                     for arch in architectures:
-                        if _kernel_version_is_supported(version, arch):
+                        if arch.kernel_version_supported(version):
                             runner.add_kernel(arch, version + ".*" + flavor.name)
             else:
                 for arch in architectures:
