@@ -6,13 +6,16 @@ import dataclasses
 import inspect
 import os
 from pathlib import Path
-from typing import Dict, Mapping, NamedTuple, Sequence
+import subprocess
+from typing import Dict, Mapping, NamedTuple, Optional, Sequence, Union
 
 from _drgn_util.platform import NORMALIZED_MACHINE_NAME
+from util import KernelVersion, out_of_date
 
 # Kernel versions that we run tests on and therefore support. Keep this in sync
 # with docs/support_matrix.rst.
 SUPPORTED_KERNEL_VERSIONS = (
+    "6.19",
     "6.18",
     "6.17",
     "6.16",
@@ -48,7 +51,7 @@ SUPPORTED_KERNEL_VERSIONS = (
     "4.9",
 )
 
-KERNEL_ORG_COMPILER_VERSION = "12.2.0"
+KERNEL_ORG_COMPILER_VERSION = "12.5.0"
 
 
 BASE_KCONFIG = """
@@ -130,6 +133,9 @@ CONFIG_CGROUPS=y
 # 3007098494be ("cgroup: add support for eBPF programs") (in v4.10)).
 CONFIG_CGROUP_NET_CLASSID=y
 
+# For ipc tests.
+CONFIG_SYSVIPC=y
+
 # For kconfig tests.
 CONFIG_IKCONFIG=m
 CONFIG_IKCONFIG_PROC=y
@@ -142,6 +148,7 @@ CONFIG_EXT4_FS=m
 CONFIG_XFS_FS=m
 
 # For mm tests.
+CONFIG_ANON_VMA_NAME=y
 CONFIG_HUGETLBFS=y
 CONFIG_MEMORY_HOTPLUG=y
 CONFIG_MEMORY_HOTREMOVE=y
@@ -153,6 +160,8 @@ CONFIG_NAMESPACES=y
 CONFIG_NUMA=y
 
 # For sched tests.
+CONFIG_CGROUP_SCHED=y
+CONFIG_FAIR_GROUP_SCHED=y
 CONFIG_SCHEDSTATS=y
 
 # For slab allocator tests.
@@ -224,8 +233,12 @@ KERNEL_FLAVORS = {
                 # and slab_def.h") (in v6.8) removed SLAB. Test this
                 # non-default SLUB option instead.
                 CONFIG_SLUB_CPU_PARTIAL=n
-                CONFIG_MODVERSIONS=y
                 CONFIG_RANDOMIZE_BASE=n
+
+                # Options that require additional metadata for kmodify.
+                CONFIG_MEM_ALLOC_PROFILING=y
+                CONFIG_MEM_ALLOC_PROFILING_ENABLED_BY_DEFAULT=y
+                CONFIG_MODVERSIONS=y
             """,
         ),
         KernelFlavor(
@@ -242,6 +255,9 @@ KERNEL_FLAVORS = {
                 # which was introduced in Linux kernel commit e240e53ae0ab
                 # ("mm, slub: add CONFIG_SLUB_TINY") (in v6.2).
                 CONFIG_SLUB_TINY=y
+                # Cover the case of disabling this feature on kernel versions
+                # that support it.
+                CONFIG_ANON_VMA_NAME=n
                 # CONFIG_PREEMPT_DYNAMIC is not set
                 CONFIG_PREEMPT_NONE=y
                 # !PREEMPTION && !SMP will also select TINY_RCU.
@@ -263,17 +279,31 @@ class Architecture:
     kernel_srcarch: str
     # Name of the architecture in Debian.
     debian_arch: str
+    # GCC target triplet on Debian.
+    debian_gcc_target: str
     # Linux kernel configuration options.
     kernel_config: str
     # Flavor-specific Linux kernel configuration options.
     kernel_flavor_configs: Mapping[str, str]
+    # Name of compiler host on
+    # https://mirrors.kernel.org/pub/tools/crosstool/.
+    kernel_org_compiler_host_name: Optional[str]
     # Name of compiler target on
     # https://mirrors.kernel.org/pub/tools/crosstool/.
-    kernel_org_compiler_name: str
+    kernel_org_compiler_target_name: str
     # Options to pass to QEMU.
     qemu_options: Sequence[str]
     # Console device when using QEMU.
     qemu_console: str
+    # Minimum supported kernel version.
+    min_kernel_version: Optional[KernelVersion] = None
+
+    def kernel_version_supported(self, version: Union[KernelVersion, str]) -> bool:
+        if self.min_kernel_version is None:
+            return True
+        if not isinstance(version, KernelVersion):
+            version = KernelVersion(version)
+        return version >= self.min_kernel_version
 
 
 ARCHITECTURES = {
@@ -284,6 +314,7 @@ ARCHITECTURES = {
             kernel_arch="arm64",
             kernel_srcarch="arm64",
             debian_arch="arm64",
+            debian_gcc_target="aarch64-linux-gnu",
             kernel_config="""
                 CONFIG_PCI_HOST_GENERIC=y
                 CONFIG_RTC_CLASS=y
@@ -302,15 +333,19 @@ ARCHITECTURES = {
                     CONFIG_ARM64_16K_PAGES=y
                 """,
             },
-            kernel_org_compiler_name="aarch64-linux",
+            kernel_org_compiler_host_name="arm64",
+            kernel_org_compiler_target_name="aarch64-linux",
             qemu_options=("-M", "virt", "-cpu", "cortex-a76"),
             qemu_console="ttyAMA0",
+            # /proc/kcore is broken on older versions.
+            min_kernel_version=KernelVersion("4.19"),
         ),
         Architecture(
             name="arm",
             kernel_arch="arm",
             kernel_srcarch="arm",
             debian_arch="armhf",
+            debian_gcc_target="arm-linux-gnueabihf",
             kernel_config="""
                 CONFIG_NR_CPUS=8
                 # Debian armhf userspace assumes EABI and VFP.
@@ -346,15 +381,19 @@ ARCHITECTURES = {
                     CONFIG_ARM_LPAE=n
                 """,
             },
-            kernel_org_compiler_name="arm-linux-gnueabi",
+            kernel_org_compiler_host_name=None,
+            kernel_org_compiler_target_name="arm-linux-gnueabi",
             qemu_options=("-M", "virt,highmem=off"),
             qemu_console="ttyAMA0",
+            # /proc/kcore is broken on older versions.
+            min_kernel_version=KernelVersion("4.19"),
         ),
         Architecture(
             name="ppc64",
             kernel_arch="powerpc",
             kernel_srcarch="powerpc",
             debian_arch="ppc64el",
+            debian_gcc_target="powerpc64le-linux-gnu",
             kernel_config="""
                 CONFIG_PPC64=y
                 CONFIG_CPU_LITTLE_ENDIAN=y
@@ -369,29 +408,80 @@ ARCHITECTURES = {
                 CONFIG_CRYPTO_AES_GCM_P10=n
             """,
             kernel_flavor_configs={},
-            kernel_org_compiler_name="powerpc64-linux",
+            kernel_org_compiler_host_name="ppc64le",
+            kernel_org_compiler_target_name="powerpc64-linux",
             qemu_options=(),
             qemu_console="hvc0",
+            # Need an implementation of
+            # linux_kernel_live_direct_mapping_fallback in libdrgn.
+            min_kernel_version=KernelVersion("4.11"),
+        ),
+        Architecture(
+            name="riscv64",
+            kernel_arch="riscv",
+            kernel_srcarch="riscv",
+            debian_arch="riscv64",
+            debian_gcc_target="riscv64-linux-gnu",
+            kernel_config="""
+                CONFIG_ARCH_RV64I=y
+                CONFIG_KERNEL_UNCOMPRESSED=y
+                CONFIG_PCI_HOST_GENERIC=y
+                CONFIG_SERIAL_8250=y
+                CONFIG_SERIAL_8250_CONSOLE=y
+                CONFIG_SERIAL_OF_PLATFORM=y
+                # For some reason, tests run significantly slower on a
+                # relocatable kernel. Additionally, as of Linux 6.19, the KASLR
+                # offset is not in VMCOREINFO:
+                # https://lore.kernel.org/linux-riscv/aXiF55y5D49gcpUg@adminpc-PowerEdge-R7525/.
+                # So, disable KASLR for now.
+                CONFIG_RANDOMIZE_BASE=n
+            """,
+            kernel_flavor_configs={},
+            kernel_org_compiler_host_name=None,
+            kernel_org_compiler_target_name="riscv64-linux",
+            qemu_options=("-M", "virt"),
+            qemu_console="ttyS0",
+            # RISC-V support was added in Linux 4.15. Linux 6.11 and older need
+            # patches to handle more relocation types in kexec_file_load(), and
+            # even after patching, 6.7-6.11 hang on kexec. Linux 6.6 and older
+            # need a backport of Linux kernel commit e0c0a7c35f67 ("riscv:
+            # select ARCH_PROC_KCORE_TEXT"). Linux 6.2 is the oldest version
+            # that I can get to boot in QEMU so far. Linux 5.15 is the oldest
+            # version that will build without additional patches.
+            min_kernel_version=KernelVersion("6.12"),
         ),
         Architecture(
             name="s390x",
             kernel_arch="s390",
             kernel_srcarch="s390",
             debian_arch="s390x",
+            debian_gcc_target="s390x-linux-gnu",
             kernel_config="""
                 # Needed for CONFIG_KEXEC_FILE.
                 CONFIG_CRYPTO_SHA256_S390=y
+
+                # Before Linux kernel commit e950d1f84d3c ("s390/percpu: Get
+                # rid of ARCH_MODULE_NEEDS_WEAK_PER_CPU") (in v6.19), memory
+                # allocation profiling triggers a bunch of
+                # WARN_ON(tag->counters) warnings on s390x. It's not worth
+                # dealing with.
+                CONFIG_MEM_ALLOC_PROFILING=n
             """,
             kernel_flavor_configs={},
-            kernel_org_compiler_name="s390-linux",
+            kernel_org_compiler_host_name=None,
+            kernel_org_compiler_target_name="s390-linux",
             qemu_options=(),
             qemu_console="ttysclp0",
+            # Need an implementation of
+            # linux_kernel_live_direct_mapping_fallback in libdrgn.
+            min_kernel_version=KernelVersion("4.11"),
         ),
         Architecture(
             name="x86_64",
             kernel_arch="x86_64",
             kernel_srcarch="x86",
             debian_arch="amd64",
+            debian_gcc_target="x86_64-linux-gnu",
             kernel_config="""
                 CONFIG_RTC_CLASS=y
                 CONFIG_RTC_DRV_CMOS=y
@@ -407,7 +497,8 @@ ARCHITECTURES = {
                     CONFIG_FRAME_POINTER_UNWINDER=y
                 """,
             },
-            kernel_org_compiler_name="x86_64-linux",
+            kernel_org_compiler_host_name="x86_64",
+            kernel_org_compiler_target_name="x86_64-linux",
             qemu_options=("-nodefaults",),
             qemu_console="ttyS0",
         ),
@@ -449,17 +540,29 @@ class Compiler(NamedTuple):
         }
 
 
+def compiler_name(host_name: str, target_name: str) -> str:
+    return f"{host_name}-gcc-{KERNEL_ORG_COMPILER_VERSION}-nolibc-{target_name}"
+
+
+def compiler_url(host_name: str, target_name: str) -> str:
+    return f"https://github.com/osandov/drgn/releases/download/vmtest-compilers/{compiler_name(host_name, target_name)}.tar.xz"
+
+
 def kconfig_localversion(arch: Architecture, flavor: KernelFlavor, version: str) -> str:
     vmtest_kernel_version = [
         # Increment the major version to rebuild every
         # architecture/flavor/version combination.
-        37,
+        40,
         # The minor version makes the default flavor the "latest" version.
         1 if flavor.name == "default" else 0,
     ]
     patch_level = 0
     # If only specific architecture/flavor/version combinations need to be
     # rebuilt, conditionally increment the patch level here.
+    if flavor.name == "alternative" and KernelVersion(version) >= KernelVersion("6.10"):
+        patch_level += 1
+        if arch.name == "s390x":
+            patch_level += 1
     if patch_level:
         vmtest_kernel_version.append(patch_level)
 
@@ -490,3 +593,21 @@ CONFIG_LOCALVERSION_AUTO=n
 
 {inspect.cleandoc(arch.kernel_flavor_configs.get(flavor.name, ""))}
 """
+
+
+def _run_autoreconf() -> None:
+    if out_of_date(
+        "libdrgn/Makefile.in", "libdrgn/Makefile.am", "libdrgn/configure.ac"
+    ) or out_of_date("libdrgn/configure", "libdrgn/configure.ac"):
+        try:
+            subprocess.check_call(["autoreconf", "-i", "libdrgn"])
+        except BaseException:
+            try:
+                os.remove("libdrgn/configure")
+            except FileNotFoundError:
+                pass
+            try:
+                os.remove("libdrgn/Makefile.in")
+            except FileNotFoundError:
+                pass
+            raise
