@@ -30,7 +30,7 @@ from vmtest.config import (
     kconfig,
     kconfig_localversion,
 )
-from vmtest.download import COMPILER_URL, Downloader
+from vmtest.download import Downloader
 
 logger = logging.getLogger(__name__)
 
@@ -187,13 +187,12 @@ _PATCHES = (
         name="4.19-kbuild-Only-add-fno-var-tracking-assignments-for-old.patch",
         versions=((None, KernelVersion("5.1")),),
     ),
-    # For aarch64, some IRQ stacks are mapped in the range [_text, _stext],
-    # which does not get mapped in /proc/vmcore. This patch is in linux-next for
-    # 6.18, and it is needed since e2a073dde9218 ("arm64: omit [_text, _stext)
-    # from permanent kernel mapping") in v5.11.
     _Patch(
         name="6.17-arm64-map-_text-_stext-virtual-address-range-non-exe.patch",
-        versions=((KernelVersion("6.9"), KernelVersion("6.18")),),
+        versions=(
+            (KernelVersion("6.13"), KernelVersion("6.17.4")),
+            (KernelVersion("6.9"), KernelVersion("6.12.54")),
+        ),
     ),
     _Patch(
         name="6.8-arm64-map-_text-_stext-virtual-address-range-non-exe.patch",
@@ -217,6 +216,47 @@ _PATCHES = (
         name="5.12-arm64-map-_text-_stext-virtual-address-range-non-exe.patch",
         versions=((KernelVersion("5.11"), KernelVersion("5.13")),),
     ),
+    _Patch(
+        name="4.9-ARM-8933-1-replace-Sun-Solaris-style-flag-on-section.patch",
+        versions=((None, KernelVersion("4.14")),),
+    ),
+    _Patch(
+        name="4.14-ARM-8933-1-replace-Sun-Solaris-style-flag-on-section.patch",
+        versions=(
+            (KernelVersion("4.14"), KernelVersion("4.14.329")),
+            (KernelVersion("4.15"), KernelVersion("4.19.298")),
+            (KernelVersion("4.20"), KernelVersion("5.4.203")),
+        ),
+    ),
+    _Patch(
+        name="powerpc-boot-Remove-leading-zero-in-label-in-udelay.patch",
+        versions=(
+            (None, KernelVersion("5.4.299")),
+            (KernelVersion("5.5"), KernelVersion("5.10")),
+        ),
+    ),
+    _Patch(
+        name="riscv-fix-vmemmap-and-vmalloc-offsets-in-proc-kcore.patch",
+        versions=((KernelVersion("6.11"), None),),
+    ),
+    _Patch(
+        name="6.10-riscv-fix-vmemmap-and-vmalloc-offsets-in-proc-kcore.patch",
+        versions=(
+            (KernelVersion("6.8"), KernelVersion("6.11")),
+            (KernelVersion("6.6.51"), KernelVersion("6.7")),
+        ),
+    ),
+    _Patch(
+        name="6.7-riscv-fix-vmemmap-and-vmalloc-offsets-in-proc-kcore.patch",
+        versions=(
+            (KernelVersion("6.7"), KernelVersion("6.8")),
+            (KernelVersion("5.17"), KernelVersion("6.6.51")),
+        ),
+    ),
+    _Patch(
+        name="5.16-riscv-fix-vmemmap-and-vmalloc-offsets-in-proc-kcore.patch",
+        versions=((KernelVersion("4.15"), KernelVersion("5.17")),),
+    ),
 )
 
 
@@ -446,6 +486,7 @@ class KBuild:
             "scripts/check-local-export",
             "scripts/gcc-goto.sh",
             "scripts/gcc-version.sh",
+            "scripts/gen-btf.sh",
             "scripts/ld-version.sh",
             "scripts/mod/modpost",
             "scripts/module-common.c",
@@ -617,12 +658,25 @@ MODULE_DESCRIPTION("Module for testing build");
             package,
         )
 
-        image_name = (
-            (await check_output("make", *make_args, "-s", "image_name", env=self._env))
-            .decode()
-            .strip()
-        )
+        if self._arch.kernel_srcarch == "riscv":
+            # `qemu-system-riscv64 -kernel` apparently can't boot a compressed
+            # kernel. Since Linux kernel commit e79dfcbfb902 ("riscv: make
+            # image compression configurable") (in v6.10),
+            # CONFIG_KERNEL_UNCOMPRESSED=y makes `make image_name` output the
+            # uncompressed image. Before that, we need a special case.
+            image_name = "Image"
+        else:
+            image_name = (
+                (
+                    await check_output(
+                        "make", *make_args, "-s", "image_name", env=self._env
+                    )
+                )
+                .decode()
+                .strip()
+            )
 
+        package.parent.mkdir(parents=True, exist_ok=True)
         with tempfile.TemporaryDirectory(
             prefix=package.name + ".tmp.", dir=package.parent
         ) as tmp_name:
@@ -690,7 +744,6 @@ MODULE_DESCRIPTION("Module for testing build");
             self._copy_module_build(modules_build_dir)
             await self._test_external_module_build(modules_build_dir)
 
-            package.parent.mkdir(parents=True, exist_ok=True)
             if format == "directory":
                 logger.info("renaming directory")
                 try:
@@ -797,7 +850,7 @@ async def main() -> None:
         nargs="?",
         default=argparse.SUPPRESS,
         type=Path,
-        help=f"download a compiler from {COMPILER_URL} to the given directory ({default_download_compiler_directory} by default) and use it to build",
+        help=f"download a compiler to the given directory ({default_download_compiler_directory} by default) and use it to build",
     )
     parser.add_argument(
         "--dump-kconfig",
