@@ -3,6 +3,7 @@
 
 import contextlib
 import enum
+import functools
 import os
 from pathlib import Path
 import re
@@ -15,7 +16,7 @@ import tempfile
 from typing import Any, Optional, Sequence, TextIO
 
 from util import nproc, out_of_date
-from vmtest.config import HOST_ARCHITECTURE, Kernel, local_kernel
+from vmtest.config import HOST_ARCHITECTURE, Kernel
 from vmtest.download import (
     DOWNLOAD_KERNEL_ARGPARSE_METAVAR,
     Downloader,
@@ -88,6 +89,9 @@ pivot_root /tmp/merged /tmp/merged/mnt
 cd /
 umount -n -l /mnt
 
+# Enable cgroup controllers.
+echo +cpu > /sys/fs/cgroup/cgroup.subtree_control 2>/dev/null || true
+
 # Load kernel modules.
 mkdir -p "/lib/modules/$RELEASE"
 mount --bind {kernel_dir} "/lib/modules/$RELEASE"
@@ -207,6 +211,18 @@ def _have_setpriv_pdeathsig() -> bool:
     return b"--pdeathsig" in help
 
 
+@functools.lru_cache(maxsize=None)
+def _have_unshare_map_auto() -> bool:
+    return (
+        subprocess.run(
+            ["unshare", "--map-root-user", "--map-auto", "true"],
+            stdout=subprocess.DEVNULL,
+            stderr=subprocess.DEVNULL,
+        ).returncode
+        == 0
+    )
+
+
 class TestKmodMode(enum.Enum):
     NONE = 0
     BUILD = 1
@@ -313,9 +329,13 @@ def run_in_vm(
                 unshare_args = [
                     "unshare",
                     "--map-root-user",
-                    "--map-users=auto",
-                    "--map-groups=auto",
                 ]
+                # --map-auto maps additional UIDs/GIDs from /etc/subuid and
+                # /etc/subgid using newuidmap/newgidmap, but that doesn't work
+                # in all environments (e.g., restricted user namespaces).
+                # Additionally, unshare only supports it since util-linux 2.38.
+                if _have_unshare_map_auto():
+                    unshare_args.append("--map-auto")
             host_virtfs_args = [
                 "-virtfs",
                 f"local,path=/,mount_tag=host,{virtfs_options}",
@@ -359,6 +379,7 @@ def run_in_vm(
             f"rootflags={_9pfs_mount_options}",
             "ro",
             f"console={kernel.arch.qemu_console},115200",
+            "oops=panic",
             "panic=-1",
             "crashkernel=256M",
             *extra_kernel_cmdline,
@@ -493,6 +514,13 @@ if __name__ == "__main__":
         type=Path,
         help="directory to use as root directory in VM (default: / for the host architecture, $directory/$arch/rootfs otherwise)",
     )
+    parser.add_argument(
+        "--use-host-rootfs",
+        choices=["never", "auto"],
+        default="auto",
+        help='if "never", use $directory/$arch/rootfs even for host architecture; '
+        'if "auto", use / for host architecture',
+    )
     parser.add_argument(
         "--qemu-options",
         metavar="OPTIONS",
@@ -543,7 +571,10 @@ if __name__ == "__main__":
         assert HOST_ARCHITECTURE is not None
         args.kernel = DownloadKernel(HOST_ARCHITECTURE, "*")
     if not hasattr(args, "root_directory"):
-        args.root_directory = None
+        if args.use_host_rootfs == "never":
+            args.root_directory = args.directory / args.kernel.arch.name / "rootfs"
+        else:
+            args.root_directory = None
     if not hasattr(args, "qemu_options"):
         args.qemu_options = []
     if not hasattr(args, "kernel_cmdline"):
@@ -555,12 +586,9 @@ if __name__ == "__main__":
     if args.test_kmod != TestKmodMode.NONE:
         downloader.download_compiler(downloader.resolve_compiler(args.kernel.arch))
 
-    if args.kernel.pattern.startswith(".") or args.kernel.pattern.startswith("/"):
-        kernel = local_kernel(args.kernel.arch, Path(args.kernel.pattern))
-    else:
-        kernel = downloader.download_kernel(
-            downloader.resolve_kernel(args.kernel.arch, args.kernel.pattern)
-        )
+    kernel = downloader.download_kernel(
+        downloader.resolve_kernel(args.kernel.arch, args.kernel.pattern)
+    )
 
     try:
         command = (
